"""
Validation helpers for web_proxy/explicit endpoint.

Each endpoint has its own validation file to keep validation logic
separate and maintainable. Use central cmdb._helpers tools for common tasks.

Auto-generated from OpenAPI specification
Customize as needed for endpoint-specific business logic.
"""

from typing import Any, TypedDict, NotRequired, Literal

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:
#
# 1. FALSE POSITIVES: Some fields marked "required" have default values,
#    meaning they're optional (filtered out by generator)
#
# 2. CONDITIONAL REQUIREMENTS: Many endpoints require EITHER field A OR field B:
#    - firewall.policy: requires (srcaddr + dstaddr) OR (srcaddr6 + dstaddr6)
#    - These conditional requirements cannot be expressed in a simple list
#
# 3. SPECIALIZED FEATURES: Fields for WAN optimization, VPN, NAT64, etc.
#    are marked "required" but only apply when using those features
#
# The REQUIRED_FIELDS list below is INFORMATIONAL ONLY and shows fields that:
# - Are marked required in the schema
# - Don't have non-empty default values
# - Aren't specialized feature fields
#
# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "interface",  # Specify outgoing interface to reach server.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "status": "disable",
    "secure-web-proxy": "disable",
    "ftp-over-http": "disable",
    "socks": "disable",
    "http-incoming-port": "",
    "http-connection-mode": "static",
    "https-incoming-port": "",
    "client-cert": "disable",
    "user-agent-detect": "enable",
    "empty-cert-action": "block",
    "ssl-dh-bits": "2048",
    "ftp-incoming-port": "",
    "socks-incoming-port": "",
    "incoming-ip": "0.0.0.0",
    "outgoing-ip": "",
    "interface-select-method": "sdwan",
    "interface": "",
    "vrf-select": -1,
    "ipv6-status": "disable",
    "incoming-ip6": "::",
    "outgoing-ip6": "",
    "strict-guest": "disable",
    "pref-dns-result": "ipv4",
    "unknown-http-version": "reject",
    "realm": "default",
    "sec-default-action": "deny",
    "https-replacement-message": "enable",
    "message-upon-server-error": "enable",
    "pac-file-server-status": "disable",
    "pac-file-url": "",
    "pac-file-server-port": "",
    "pac-file-through-https": "disable",
    "pac-file-name": "proxy.pac",
    "pac-file-data": "",
    "ssl-algorithm": "low",
    "trace-auth-no-rsp": "disable",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "status": "option",  # Enable/disable the explicit Web proxy for HTTP and HTTPS ses
    "secure-web-proxy": "option",  # Enable/disable/require the secure web proxy for HTTP and HTT
    "ftp-over-http": "option",  # Enable to proxy FTP-over-HTTP sessions sent from a web brows
    "socks": "option",  # Enable/disable the SOCKS proxy.
    "http-incoming-port": "user",  # Accept incoming HTTP requests on one or more ports (0 - 6553
    "http-connection-mode": "option",  # HTTP connection mode (default = static).
    "https-incoming-port": "user",  # Accept incoming HTTPS requests on one or more ports (0 - 655
    "secure-web-proxy-cert": "string",  # Name of certificates for secure web proxy.
    "client-cert": "option",  # Enable/disable to request client certificate.
    "user-agent-detect": "option",  # Enable/disable to detect device type by HTTP user-agent if n
    "empty-cert-action": "option",  # Action of an empty client certificate.
    "ssl-dh-bits": "option",  # Bit-size of Diffie-Hellman (DH) prime used in DHE-RSA negoti
    "ftp-incoming-port": "user",  # Accept incoming FTP-over-HTTP requests on one or more ports 
    "socks-incoming-port": "user",  # Accept incoming SOCKS proxy requests on one or more ports (0
    "incoming-ip": "ipv4-address-any",  # Restrict the explicit HTTP proxy to only accept sessions fro
    "outgoing-ip": "ipv4-address-any",  # Outgoing HTTP requests will have this IP address as their so
    "interface-select-method": "option",  # Specify how to select outgoing interface to reach server.
    "interface": "string",  # Specify outgoing interface to reach server.
    "vrf-select": "integer",  # VRF ID used for connection to server.
    "ipv6-status": "option",  # Enable/disable allowing an IPv6 web proxy destination in pol
    "incoming-ip6": "ipv6-address",  # Restrict the explicit web proxy to only accept sessions from
    "outgoing-ip6": "ipv6-address",  # Outgoing HTTP requests will leave this IPv6. Multiple interf
    "strict-guest": "option",  # Enable/disable strict guest user checking by the explicit we
    "pref-dns-result": "option",  # Prefer resolving addresses using the configured IPv4 or IPv6
    "unknown-http-version": "option",  # How to handle HTTP sessions that do not comply with HTTP 0.9
    "realm": "string",  # Authentication realm used to identify the explicit web proxy
    "sec-default-action": "option",  # Accept or deny explicit web proxy sessions when no web proxy
    "https-replacement-message": "option",  # Enable/disable sending the client a replacement message for 
    "message-upon-server-error": "option",  # Enable/disable displaying a replacement message when a serve
    "pac-file-server-status": "option",  # Enable/disable Proxy Auto-Configuration (PAC) for users of t
    "pac-file-url": "user",  # PAC file access URL.
    "pac-file-server-port": "user",  # Port number that PAC traffic from client web browsers uses t
    "pac-file-through-https": "option",  # Enable/disable to get Proxy Auto-Configuration (PAC) through
    "pac-file-name": "string",  # Pac file name.
    "pac-file-data": "user",  # PAC file contents enclosed in quotes (maximum of 256K bytes)
    "pac-policy": "string",  # PAC policies.
    "ssl-algorithm": "option",  # Relative strength of encryption algorithms accepted in HTTPS
    "trace-auth-no-rsp": "option",  # Enable/disable logging timed-out authentication requests.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "status": "Enable/disable the explicit Web proxy for HTTP and HTTPS session.",
    "secure-web-proxy": "Enable/disable/require the secure web proxy for HTTP and HTTPS session.",
    "ftp-over-http": "Enable to proxy FTP-over-HTTP sessions sent from a web browser.",
    "socks": "Enable/disable the SOCKS proxy.",
    "http-incoming-port": "Accept incoming HTTP requests on one or more ports (0 - 65535, default = 8080).",
    "http-connection-mode": "HTTP connection mode (default = static).",
    "https-incoming-port": "Accept incoming HTTPS requests on one or more ports (0 - 65535, default = 0, use the same as HTTP).",
    "secure-web-proxy-cert": "Name of certificates for secure web proxy.",
    "client-cert": "Enable/disable to request client certificate.",
    "user-agent-detect": "Enable/disable to detect device type by HTTP user-agent if no client certificate provided.",
    "empty-cert-action": "Action of an empty client certificate.",
    "ssl-dh-bits": "Bit-size of Diffie-Hellman (DH) prime used in DHE-RSA negotiation (default = 2048).",
    "ftp-incoming-port": "Accept incoming FTP-over-HTTP requests on one or more ports (0 - 65535, default = 0; use the same as HTTP).",
    "socks-incoming-port": "Accept incoming SOCKS proxy requests on one or more ports (0 - 65535, default = 0; use the same as HTTP).",
    "incoming-ip": "Restrict the explicit HTTP proxy to only accept sessions from this IP address. An interface must have this IP address.",
    "outgoing-ip": "Outgoing HTTP requests will have this IP address as their source address. An interface must have this IP address.",
    "interface-select-method": "Specify how to select outgoing interface to reach server.",
    "interface": "Specify outgoing interface to reach server.",
    "vrf-select": "VRF ID used for connection to server.",
    "ipv6-status": "Enable/disable allowing an IPv6 web proxy destination in policies and all IPv6 related entries in this command.",
    "incoming-ip6": "Restrict the explicit web proxy to only accept sessions from this IPv6 address. An interface must have this IPv6 address.",
    "outgoing-ip6": "Outgoing HTTP requests will leave this IPv6. Multiple interfaces can be specified. Interfaces must have these IPv6 addresses.",
    "strict-guest": "Enable/disable strict guest user checking by the explicit web proxy.",
    "pref-dns-result": "Prefer resolving addresses using the configured IPv4 or IPv6 DNS server (default = ipv4).",
    "unknown-http-version": "How to handle HTTP sessions that do not comply with HTTP 0.9, 1.0, or 1.1.",
    "realm": "Authentication realm used to identify the explicit web proxy (maximum of 63 characters).",
    "sec-default-action": "Accept or deny explicit web proxy sessions when no web proxy firewall policy exists.",
    "https-replacement-message": "Enable/disable sending the client a replacement message for HTTPS requests.",
    "message-upon-server-error": "Enable/disable displaying a replacement message when a server error is detected.",
    "pac-file-server-status": "Enable/disable Proxy Auto-Configuration (PAC) for users of this explicit proxy profile.",
    "pac-file-url": "PAC file access URL.",
    "pac-file-server-port": "Port number that PAC traffic from client web browsers uses to connect to the explicit web proxy (0 - 65535, default = 0; use the same as HTTP).",
    "pac-file-through-https": "Enable/disable to get Proxy Auto-Configuration (PAC) through HTTPS.",
    "pac-file-name": "Pac file name.",
    "pac-file-data": "PAC file contents enclosed in quotes (maximum of 256K bytes).",
    "pac-policy": "PAC policies.",
    "ssl-algorithm": "Relative strength of encryption algorithms accepted in HTTPS deep scan: high, medium, or low.",
    "trace-auth-no-rsp": "Enable/disable logging timed-out authentication requests.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "interface": {"type": "string", "max_length": 15},
    "vrf-select": {"type": "integer", "min": 0, "max": 511},
    "realm": {"type": "string", "max_length": 63},
    "pac-file-name": {"type": "string", "max_length": 63},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "secure-web-proxy-cert": {
        "name": {
            "type": "string",
            "help": "Certificate list.",
            "default": "Fortinet_SSL",
            "max_length": 79,
        },
    },
    "pac-policy": {
        "policyid": {
            "type": "integer",
            "help": "Policy ID.",
            "required": True,
            "default": 0,
            "min_value": 1,
            "max_value": 100,
        },
        "status": {
            "type": "option",
            "help": "Enable/disable policy.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "srcaddr": {
            "type": "string",
            "help": "Source address objects.",
            "required": True,
        },
        "srcaddr6": {
            "type": "string",
            "help": "Source address6 objects.",
        },
        "dstaddr": {
            "type": "string",
            "help": "Destination address objects.",
            "required": True,
        },
        "pac-file-name": {
            "type": "string",
            "help": "Pac file name.",
            "required": True,
            "default": "proxy.pac",
            "max_length": 63,
        },
        "pac-file-data": {
            "type": "user",
            "help": "PAC file contents enclosed in quotes (maximum of 256K bytes).",
            "default": "",
        },
        "comments": {
            "type": "var-string",
            "help": "Optional comments.",
            "max_length": 1023,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_STATUS = [
    "enable",
    "disable",
]
VALID_BODY_SECURE_WEB_PROXY = [
    "disable",
    "enable",
    "secure",
]
VALID_BODY_FTP_OVER_HTTP = [
    "enable",
    "disable",
]
VALID_BODY_SOCKS = [
    "enable",
    "disable",
]
VALID_BODY_HTTP_CONNECTION_MODE = [
    "static",
    "multiplex",
    "serverpool",
]
VALID_BODY_CLIENT_CERT = [
    "disable",
    "enable",
]
VALID_BODY_USER_AGENT_DETECT = [
    "disable",
    "enable",
]
VALID_BODY_EMPTY_CERT_ACTION = [
    "accept",
    "block",
    "accept-unmanageable",
]
VALID_BODY_SSL_DH_BITS = [
    "768",
    "1024",
    "1536",
    "2048",
]
VALID_BODY_INTERFACE_SELECT_METHOD = [
    "sdwan",
    "specify",
]
VALID_BODY_IPV6_STATUS = [
    "enable",
    "disable",
]
VALID_BODY_STRICT_GUEST = [
    "enable",
    "disable",
]
VALID_BODY_PREF_DNS_RESULT = [
    "ipv4",
    "ipv6",
    "ipv4-strict",
    "ipv6-strict",
]
VALID_BODY_UNKNOWN_HTTP_VERSION = [
    "reject",
    "best-effort",
]
VALID_BODY_SEC_DEFAULT_ACTION = [
    "accept",
    "deny",
]
VALID_BODY_HTTPS_REPLACEMENT_MESSAGE = [
    "enable",
    "disable",
]
VALID_BODY_MESSAGE_UPON_SERVER_ERROR = [
    "enable",
    "disable",
]
VALID_BODY_PAC_FILE_SERVER_STATUS = [
    "enable",
    "disable",
]
VALID_BODY_PAC_FILE_THROUGH_HTTPS = [
    "enable",
    "disable",
]
VALID_BODY_SSL_ALGORITHM = [
    "high",
    "medium",
    "low",
]
VALID_BODY_TRACE_AUTH_NO_RSP = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_web_proxy_explicit_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """
    Validate GET request parameters for web_proxy/explicit.

    Args:
        attr: Attribute filter (optional)
        filters: Additional filter parameters
        **params: Other query parameters

    Returns:
        Tuple of (is_valid, error_message)

    Examples:
        >>> # Valid - Get all items
        >>> is_valid, error = validate_web_proxy_explicit_get()
        >>> assert is_valid == True
        
        
        >>> # Valid - With filters
        >>> is_valid, error = validate_web_proxy_explicit_get(
        ...     filters={"format": "name|type"}
        ... )
        >>> assert is_valid == True
    """
    # Validate query parameters if present
    if "action" in params:
        value = params.get("action")
        if value and value not in VALID_QUERY_ACTION:
            return (
                False,
                f"Invalid query parameter 'action'='{value}'. Must be one of: {', '.join(VALID_QUERY_ACTION)}",
            )

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_required_fields(payload: dict) -> tuple[bool, str | None]:
    """
    Validate required fields for web_proxy/explicit.

    This validator checks:
    1. Always-required fields are present
    2. Mutually exclusive groups have at least one field

    Args:
        payload: The request payload to validate

    Returns:
        Tuple of (is_valid, error_message)

    Example:
        >>> payload = {"name": "test"}
        >>> is_valid, error = validate_required_fields(payload)
    """
    # Check always-required fields
    missing_fields = []
    for field in REQUIRED_FIELDS:
        if field not in payload:
            missing_fields.append(field)
    
    if missing_fields:
        # Build enhanced error message
        error_parts = [f"Missing required field(s): {', '.join(missing_fields)}"]
        
        # Add descriptions for first few missing fields
        for field in missing_fields[:3]:
            desc = FIELD_DESCRIPTIONS.get(field)
            if desc:
                error_parts.append(f"  • {field}: {desc}")
        
        if len(missing_fields) > 3:
            error_parts.append(f"  ... and {len(missing_fields) - 3} more")
        
        return (False, "\n".join(error_parts))

    return (True, None)


def validate_web_proxy_explicit_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """
    Validate POST request to create new web_proxy/explicit object.

    This validator performs two-stage validation:
    1. Required fields check (schema-based)
    2. Field value validation (enums, ranges, formats)

    Args:
        payload: Request body data with configuration
        **params: Query parameters (vdom, etc.)

    Returns:
        Tuple of (is_valid, error_message)
        - is_valid: True if payload is valid, False otherwise
        - error_message: None if valid, detailed error string if invalid

    Examples:
        >>> # ✅ Valid - Minimal required fields
        >>> payload = {
        ...     "interface": True,  # Specify outgoing interface to reach server.
        ... }
        >>> is_valid, error = validate_web_proxy_explicit_post(payload)
        >>> assert is_valid == True
        
        >>> # ✅ Valid - With enum field
        >>> payload = {
        ...     "interface": True,
        ...     "status": "enable",  # Valid enum value
        ... }
        >>> is_valid, error = validate_web_proxy_explicit_post(payload)
        >>> assert is_valid == True
        
        >>> # ❌ Invalid - Wrong enum value
        >>> payload["status"] = "invalid-value"
        >>> is_valid, error = validate_web_proxy_explicit_post(payload)
        >>> assert is_valid == False
        >>> assert "Invalid value" in error
        
        >>> # ❌ Invalid - Missing required field
        >>> payload = {}  # Empty payload
        >>> is_valid, error = validate_web_proxy_explicit_post(payload)
        >>> assert is_valid == False
        >>> assert "Missing required field" in error
    """
    # Step 1: Validate required fields
    is_valid, error = validate_required_fields(payload)
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values
    if "status" in payload:
        value = payload["status"]
        if value not in VALID_BODY_STATUS:
            desc = FIELD_DESCRIPTIONS.get("status", "")
            error_msg = f"Invalid value for 'status': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_STATUS)}"
            error_msg += f"\n  → Example: status='{{ VALID_BODY_STATUS[0] }}'"
            return (False, error_msg)
    if "secure-web-proxy" in payload:
        value = payload["secure-web-proxy"]
        if value not in VALID_BODY_SECURE_WEB_PROXY:
            desc = FIELD_DESCRIPTIONS.get("secure-web-proxy", "")
            error_msg = f"Invalid value for 'secure-web-proxy': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SECURE_WEB_PROXY)}"
            error_msg += f"\n  → Example: secure-web-proxy='{{ VALID_BODY_SECURE_WEB_PROXY[0] }}'"
            return (False, error_msg)
    if "ftp-over-http" in payload:
        value = payload["ftp-over-http"]
        if value not in VALID_BODY_FTP_OVER_HTTP:
            desc = FIELD_DESCRIPTIONS.get("ftp-over-http", "")
            error_msg = f"Invalid value for 'ftp-over-http': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_FTP_OVER_HTTP)}"
            error_msg += f"\n  → Example: ftp-over-http='{{ VALID_BODY_FTP_OVER_HTTP[0] }}'"
            return (False, error_msg)
    if "socks" in payload:
        value = payload["socks"]
        if value not in VALID_BODY_SOCKS:
            desc = FIELD_DESCRIPTIONS.get("socks", "")
            error_msg = f"Invalid value for 'socks': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SOCKS)}"
            error_msg += f"\n  → Example: socks='{{ VALID_BODY_SOCKS[0] }}'"
            return (False, error_msg)
    if "http-connection-mode" in payload:
        value = payload["http-connection-mode"]
        if value not in VALID_BODY_HTTP_CONNECTION_MODE:
            desc = FIELD_DESCRIPTIONS.get("http-connection-mode", "")
            error_msg = f"Invalid value for 'http-connection-mode': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_HTTP_CONNECTION_MODE)}"
            error_msg += f"\n  → Example: http-connection-mode='{{ VALID_BODY_HTTP_CONNECTION_MODE[0] }}'"
            return (False, error_msg)
    if "client-cert" in payload:
        value = payload["client-cert"]
        if value not in VALID_BODY_CLIENT_CERT:
            desc = FIELD_DESCRIPTIONS.get("client-cert", "")
            error_msg = f"Invalid value for 'client-cert': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_CLIENT_CERT)}"
            error_msg += f"\n  → Example: client-cert='{{ VALID_BODY_CLIENT_CERT[0] }}'"
            return (False, error_msg)
    if "user-agent-detect" in payload:
        value = payload["user-agent-detect"]
        if value not in VALID_BODY_USER_AGENT_DETECT:
            desc = FIELD_DESCRIPTIONS.get("user-agent-detect", "")
            error_msg = f"Invalid value for 'user-agent-detect': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_USER_AGENT_DETECT)}"
            error_msg += f"\n  → Example: user-agent-detect='{{ VALID_BODY_USER_AGENT_DETECT[0] }}'"
            return (False, error_msg)
    if "empty-cert-action" in payload:
        value = payload["empty-cert-action"]
        if value not in VALID_BODY_EMPTY_CERT_ACTION:
            desc = FIELD_DESCRIPTIONS.get("empty-cert-action", "")
            error_msg = f"Invalid value for 'empty-cert-action': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_EMPTY_CERT_ACTION)}"
            error_msg += f"\n  → Example: empty-cert-action='{{ VALID_BODY_EMPTY_CERT_ACTION[0] }}'"
            return (False, error_msg)
    if "ssl-dh-bits" in payload:
        value = payload["ssl-dh-bits"]
        if value not in VALID_BODY_SSL_DH_BITS:
            desc = FIELD_DESCRIPTIONS.get("ssl-dh-bits", "")
            error_msg = f"Invalid value for 'ssl-dh-bits': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SSL_DH_BITS)}"
            error_msg += f"\n  → Example: ssl-dh-bits='{{ VALID_BODY_SSL_DH_BITS[0] }}'"
            return (False, error_msg)
    if "interface-select-method" in payload:
        value = payload["interface-select-method"]
        if value not in VALID_BODY_INTERFACE_SELECT_METHOD:
            desc = FIELD_DESCRIPTIONS.get("interface-select-method", "")
            error_msg = f"Invalid value for 'interface-select-method': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_INTERFACE_SELECT_METHOD)}"
            error_msg += f"\n  → Example: interface-select-method='{{ VALID_BODY_INTERFACE_SELECT_METHOD[0] }}'"
            return (False, error_msg)
    if "ipv6-status" in payload:
        value = payload["ipv6-status"]
        if value not in VALID_BODY_IPV6_STATUS:
            desc = FIELD_DESCRIPTIONS.get("ipv6-status", "")
            error_msg = f"Invalid value for 'ipv6-status': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_IPV6_STATUS)}"
            error_msg += f"\n  → Example: ipv6-status='{{ VALID_BODY_IPV6_STATUS[0] }}'"
            return (False, error_msg)
    if "strict-guest" in payload:
        value = payload["strict-guest"]
        if value not in VALID_BODY_STRICT_GUEST:
            desc = FIELD_DESCRIPTIONS.get("strict-guest", "")
            error_msg = f"Invalid value for 'strict-guest': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_STRICT_GUEST)}"
            error_msg += f"\n  → Example: strict-guest='{{ VALID_BODY_STRICT_GUEST[0] }}'"
            return (False, error_msg)
    if "pref-dns-result" in payload:
        value = payload["pref-dns-result"]
        if value not in VALID_BODY_PREF_DNS_RESULT:
            desc = FIELD_DESCRIPTIONS.get("pref-dns-result", "")
            error_msg = f"Invalid value for 'pref-dns-result': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_PREF_DNS_RESULT)}"
            error_msg += f"\n  → Example: pref-dns-result='{{ VALID_BODY_PREF_DNS_RESULT[0] }}'"
            return (False, error_msg)
    if "unknown-http-version" in payload:
        value = payload["unknown-http-version"]
        if value not in VALID_BODY_UNKNOWN_HTTP_VERSION:
            desc = FIELD_DESCRIPTIONS.get("unknown-http-version", "")
            error_msg = f"Invalid value for 'unknown-http-version': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_UNKNOWN_HTTP_VERSION)}"
            error_msg += f"\n  → Example: unknown-http-version='{{ VALID_BODY_UNKNOWN_HTTP_VERSION[0] }}'"
            return (False, error_msg)
    if "sec-default-action" in payload:
        value = payload["sec-default-action"]
        if value not in VALID_BODY_SEC_DEFAULT_ACTION:
            desc = FIELD_DESCRIPTIONS.get("sec-default-action", "")
            error_msg = f"Invalid value for 'sec-default-action': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SEC_DEFAULT_ACTION)}"
            error_msg += f"\n  → Example: sec-default-action='{{ VALID_BODY_SEC_DEFAULT_ACTION[0] }}'"
            return (False, error_msg)
    if "https-replacement-message" in payload:
        value = payload["https-replacement-message"]
        if value not in VALID_BODY_HTTPS_REPLACEMENT_MESSAGE:
            desc = FIELD_DESCRIPTIONS.get("https-replacement-message", "")
            error_msg = f"Invalid value for 'https-replacement-message': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_HTTPS_REPLACEMENT_MESSAGE)}"
            error_msg += f"\n  → Example: https-replacement-message='{{ VALID_BODY_HTTPS_REPLACEMENT_MESSAGE[0] }}'"
            return (False, error_msg)
    if "message-upon-server-error" in payload:
        value = payload["message-upon-server-error"]
        if value not in VALID_BODY_MESSAGE_UPON_SERVER_ERROR:
            desc = FIELD_DESCRIPTIONS.get("message-upon-server-error", "")
            error_msg = f"Invalid value for 'message-upon-server-error': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_MESSAGE_UPON_SERVER_ERROR)}"
            error_msg += f"\n  → Example: message-upon-server-error='{{ VALID_BODY_MESSAGE_UPON_SERVER_ERROR[0] }}'"
            return (False, error_msg)
    if "pac-file-server-status" in payload:
        value = payload["pac-file-server-status"]
        if value not in VALID_BODY_PAC_FILE_SERVER_STATUS:
            desc = FIELD_DESCRIPTIONS.get("pac-file-server-status", "")
            error_msg = f"Invalid value for 'pac-file-server-status': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_PAC_FILE_SERVER_STATUS)}"
            error_msg += f"\n  → Example: pac-file-server-status='{{ VALID_BODY_PAC_FILE_SERVER_STATUS[0] }}'"
            return (False, error_msg)
    if "pac-file-through-https" in payload:
        value = payload["pac-file-through-https"]
        if value not in VALID_BODY_PAC_FILE_THROUGH_HTTPS:
            desc = FIELD_DESCRIPTIONS.get("pac-file-through-https", "")
            error_msg = f"Invalid value for 'pac-file-through-https': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_PAC_FILE_THROUGH_HTTPS)}"
            error_msg += f"\n  → Example: pac-file-through-https='{{ VALID_BODY_PAC_FILE_THROUGH_HTTPS[0] }}'"
            return (False, error_msg)
    if "ssl-algorithm" in payload:
        value = payload["ssl-algorithm"]
        if value not in VALID_BODY_SSL_ALGORITHM:
            desc = FIELD_DESCRIPTIONS.get("ssl-algorithm", "")
            error_msg = f"Invalid value for 'ssl-algorithm': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SSL_ALGORITHM)}"
            error_msg += f"\n  → Example: ssl-algorithm='{{ VALID_BODY_SSL_ALGORITHM[0] }}'"
            return (False, error_msg)
    if "trace-auth-no-rsp" in payload:
        value = payload["trace-auth-no-rsp"]
        if value not in VALID_BODY_TRACE_AUTH_NO_RSP:
            desc = FIELD_DESCRIPTIONS.get("trace-auth-no-rsp", "")
            error_msg = f"Invalid value for 'trace-auth-no-rsp': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_TRACE_AUTH_NO_RSP)}"
            error_msg += f"\n  → Example: trace-auth-no-rsp='{{ VALID_BODY_TRACE_AUTH_NO_RSP[0] }}'"
            return (False, error_msg)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_web_proxy_explicit_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """
    Validate PUT request to update web_proxy/explicit.

    Args:
        payload: Request body data
        **params: Query parameters

    Returns:
        Tuple of (is_valid, error_message)

    Example:
        >>> payload = {"name": "updated_item"}
        >>> is_valid, error = validate_web_proxy_explicit_put(payload)
    """
    # Step 1: Validate enum values
    if "status" in payload:
        value = payload["status"]
        if value not in VALID_BODY_STATUS:
            return (
                False,
                f"Invalid value for 'status'='{value}'. Must be one of: {', '.join(VALID_BODY_STATUS)}",
            )
    if "secure-web-proxy" in payload:
        value = payload["secure-web-proxy"]
        if value not in VALID_BODY_SECURE_WEB_PROXY:
            return (
                False,
                f"Invalid value for 'secure-web-proxy'='{value}'. Must be one of: {', '.join(VALID_BODY_SECURE_WEB_PROXY)}",
            )
    if "ftp-over-http" in payload:
        value = payload["ftp-over-http"]
        if value not in VALID_BODY_FTP_OVER_HTTP:
            return (
                False,
                f"Invalid value for 'ftp-over-http'='{value}'. Must be one of: {', '.join(VALID_BODY_FTP_OVER_HTTP)}",
            )
    if "socks" in payload:
        value = payload["socks"]
        if value not in VALID_BODY_SOCKS:
            return (
                False,
                f"Invalid value for 'socks'='{value}'. Must be one of: {', '.join(VALID_BODY_SOCKS)}",
            )
    if "http-connection-mode" in payload:
        value = payload["http-connection-mode"]
        if value not in VALID_BODY_HTTP_CONNECTION_MODE:
            return (
                False,
                f"Invalid value for 'http-connection-mode'='{value}'. Must be one of: {', '.join(VALID_BODY_HTTP_CONNECTION_MODE)}",
            )
    if "client-cert" in payload:
        value = payload["client-cert"]
        if value not in VALID_BODY_CLIENT_CERT:
            return (
                False,
                f"Invalid value for 'client-cert'='{value}'. Must be one of: {', '.join(VALID_BODY_CLIENT_CERT)}",
            )
    if "user-agent-detect" in payload:
        value = payload["user-agent-detect"]
        if value not in VALID_BODY_USER_AGENT_DETECT:
            return (
                False,
                f"Invalid value for 'user-agent-detect'='{value}'. Must be one of: {', '.join(VALID_BODY_USER_AGENT_DETECT)}",
            )
    if "empty-cert-action" in payload:
        value = payload["empty-cert-action"]
        if value not in VALID_BODY_EMPTY_CERT_ACTION:
            return (
                False,
                f"Invalid value for 'empty-cert-action'='{value}'. Must be one of: {', '.join(VALID_BODY_EMPTY_CERT_ACTION)}",
            )
    if "ssl-dh-bits" in payload:
        value = payload["ssl-dh-bits"]
        if value not in VALID_BODY_SSL_DH_BITS:
            return (
                False,
                f"Invalid value for 'ssl-dh-bits'='{value}'. Must be one of: {', '.join(VALID_BODY_SSL_DH_BITS)}",
            )
    if "interface-select-method" in payload:
        value = payload["interface-select-method"]
        if value not in VALID_BODY_INTERFACE_SELECT_METHOD:
            return (
                False,
                f"Invalid value for 'interface-select-method'='{value}'. Must be one of: {', '.join(VALID_BODY_INTERFACE_SELECT_METHOD)}",
            )
    if "ipv6-status" in payload:
        value = payload["ipv6-status"]
        if value not in VALID_BODY_IPV6_STATUS:
            return (
                False,
                f"Invalid value for 'ipv6-status'='{value}'. Must be one of: {', '.join(VALID_BODY_IPV6_STATUS)}",
            )
    if "strict-guest" in payload:
        value = payload["strict-guest"]
        if value not in VALID_BODY_STRICT_GUEST:
            return (
                False,
                f"Invalid value for 'strict-guest'='{value}'. Must be one of: {', '.join(VALID_BODY_STRICT_GUEST)}",
            )
    if "pref-dns-result" in payload:
        value = payload["pref-dns-result"]
        if value not in VALID_BODY_PREF_DNS_RESULT:
            return (
                False,
                f"Invalid value for 'pref-dns-result'='{value}'. Must be one of: {', '.join(VALID_BODY_PREF_DNS_RESULT)}",
            )
    if "unknown-http-version" in payload:
        value = payload["unknown-http-version"]
        if value not in VALID_BODY_UNKNOWN_HTTP_VERSION:
            return (
                False,
                f"Invalid value for 'unknown-http-version'='{value}'. Must be one of: {', '.join(VALID_BODY_UNKNOWN_HTTP_VERSION)}",
            )
    if "sec-default-action" in payload:
        value = payload["sec-default-action"]
        if value not in VALID_BODY_SEC_DEFAULT_ACTION:
            return (
                False,
                f"Invalid value for 'sec-default-action'='{value}'. Must be one of: {', '.join(VALID_BODY_SEC_DEFAULT_ACTION)}",
            )
    if "https-replacement-message" in payload:
        value = payload["https-replacement-message"]
        if value not in VALID_BODY_HTTPS_REPLACEMENT_MESSAGE:
            return (
                False,
                f"Invalid value for 'https-replacement-message'='{value}'. Must be one of: {', '.join(VALID_BODY_HTTPS_REPLACEMENT_MESSAGE)}",
            )
    if "message-upon-server-error" in payload:
        value = payload["message-upon-server-error"]
        if value not in VALID_BODY_MESSAGE_UPON_SERVER_ERROR:
            return (
                False,
                f"Invalid value for 'message-upon-server-error'='{value}'. Must be one of: {', '.join(VALID_BODY_MESSAGE_UPON_SERVER_ERROR)}",
            )
    if "pac-file-server-status" in payload:
        value = payload["pac-file-server-status"]
        if value not in VALID_BODY_PAC_FILE_SERVER_STATUS:
            return (
                False,
                f"Invalid value for 'pac-file-server-status'='{value}'. Must be one of: {', '.join(VALID_BODY_PAC_FILE_SERVER_STATUS)}",
            )
    if "pac-file-through-https" in payload:
        value = payload["pac-file-through-https"]
        if value not in VALID_BODY_PAC_FILE_THROUGH_HTTPS:
            return (
                False,
                f"Invalid value for 'pac-file-through-https'='{value}'. Must be one of: {', '.join(VALID_BODY_PAC_FILE_THROUGH_HTTPS)}",
            )
    if "ssl-algorithm" in payload:
        value = payload["ssl-algorithm"]
        if value not in VALID_BODY_SSL_ALGORITHM:
            return (
                False,
                f"Invalid value for 'ssl-algorithm'='{value}'. Must be one of: {', '.join(VALID_BODY_SSL_ALGORITHM)}",
            )
    if "trace-auth-no-rsp" in payload:
        value = payload["trace-auth-no-rsp"]
        if value not in VALID_BODY_TRACE_AUTH_NO_RSP:
            return (
                False,
                f"Invalid value for 'trace-auth-no-rsp'='{value}'. Must be one of: {', '.join(VALID_BODY_TRACE_AUTH_NO_RSP)}",
            )

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Provide programmatic access to field metadata for IDE autocomplete,
# documentation generation, and dynamic validation
# ============================================================================


def get_field_description(field_name: str) -> str | None:
    """
    Get description/help text for a field.

    Args:
        field_name: Name of the field

    Returns:
        Description text or None if field doesn't exist

    Example:
        >>> desc = get_field_description("name")
        >>> print(desc)
    """
    return FIELD_DESCRIPTIONS.get(field_name)


def get_field_type(field_name: str) -> str | None:
    """
    Get the type of a field.

    Args:
        field_name: Name of the field

    Returns:
        Field type (e.g., "string", "integer", "option") or None

    Example:
        >>> field_type = get_field_type("status")
        >>> print(field_type)  # "option"
    """
    return FIELD_TYPES.get(field_name)


def get_field_constraints(field_name: str) -> dict[str, Any] | None:
    """
    Get constraints for a field (min/max values, string length).

    Args:
        field_name: Name of the field

    Returns:
        Constraint dict or None

    Example:
        >>> constraints = get_field_constraints("port")
        >>> print(constraints)  # {"type": "integer", "min": 1, "max": 65535}
    """
    return FIELD_CONSTRAINTS.get(field_name)


def get_field_default(field_name: str) -> Any | None:
    """
    Get default value for a field.

    Args:
        field_name: Name of the field

    Returns:
        Default value or None if no default

    Example:
        >>> default = get_field_default("status")
        >>> print(default)  # "enable"
    """
    return FIELDS_WITH_DEFAULTS.get(field_name)


def get_field_options(field_name: str) -> list[str] | None:
    """
    Get valid enum options for a field.

    Args:
        field_name: Name of the field

    Returns:
        List of valid values or None if not an enum field

    Example:
        >>> options = get_field_options("status")
        >>> print(options)  # ["enable", "disable"]
    """
    # Construct the constant name from field name
    # Replace all non-alphanumeric characters with underscores for valid Python identifiers
    import re
    safe_name = re.sub(r'[^a-zA-Z0-9]', '_', field_name)
    constant_name = f"VALID_BODY_{safe_name.upper()}"
    return globals().get(constant_name)


def get_nested_schema(field_name: str) -> dict[str, Any] | None:
    """
    Get schema for nested table/list fields.

    Args:
        field_name: Name of the parent field

    Returns:
        Dict mapping child field names to their metadata

    Example:
        >>> nested = get_nested_schema("members")
        >>> if nested:
        ...     for child_field, child_meta in nested.items():
        ...         print(f"{child_field}: {child_meta['type']}")
    """
    return NESTED_SCHEMAS.get(field_name)


def get_all_fields() -> list[str]:
    """
    Get list of all field names.

    Returns:
        List of all field names in the schema

    Example:
        >>> fields = get_all_fields()
        >>> print(len(fields))
    """
    return list(FIELD_TYPES.keys())


def get_field_metadata(field_name: str) -> dict[str, Any] | None:
    """
    Get complete metadata for a field (type, description, constraints, defaults, options).

    Args:
        field_name: Name of the field

    Returns:
        Dict with all available metadata or None if field doesn't exist

    Example:
        >>> meta = get_field_metadata("status")
        >>> print(meta)
        >>> # {
        >>> #   "type": "option",
        >>> #   "description": "Enable/disable this feature",
        >>> #   "default": "enable",
        >>> #   "options": ["enable", "disable"]
        >>> # }
    """
    if field_name not in FIELD_TYPES:
        return None

    metadata = {
        "name": field_name,
        "type": FIELD_TYPES[field_name],
    }

    # Add description if available
    if field_name in FIELD_DESCRIPTIONS:
        metadata["description"] = FIELD_DESCRIPTIONS[field_name]

    # Add constraints if available
    if field_name in FIELD_CONSTRAINTS:
        metadata["constraints"] = FIELD_CONSTRAINTS[field_name]

    # Add default if available
    if field_name in FIELDS_WITH_DEFAULTS:
        metadata["default"] = FIELDS_WITH_DEFAULTS[field_name]

    # Add required flag
    metadata["required"] = field_name in REQUIRED_FIELDS

    # Add options if available
    options = get_field_options(field_name)
    if options:
        metadata["options"] = options

    # Add nested schema if available
    nested = get_nested_schema(field_name)
    if nested:
        metadata["nested_schema"] = nested

    return metadata


def validate_field_value(field_name: str, value: Any) -> tuple[bool, str | None]:
    """
    Validate a single field value against its constraints.

    Args:
        field_name: Name of the field
        value: Value to validate

    Returns:
        Tuple of (is_valid, error_message)

    Example:
        >>> is_valid, error = validate_field_value("status", "enable")
        >>> if not is_valid:
        ...     print(error)
    """
    # Get field metadata
    field_type = get_field_type(field_name)
    if field_type is None:
        return (False, f"Unknown field: '{field_name}' (not defined in schema)")

    # Get field description for better error context
    description = get_field_description(field_name)

    # Validate enum values
    options = get_field_options(field_name)
    if options and value not in options:
        error_msg = f"Invalid value for '{field_name}': {repr(value)}"
        if description:
            error_msg += f"\n  → Description: {description}"
        error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in options)}"
        if options:
            error_msg += f"\n  → Example: {field_name}={repr(options[0])}"
        return (False, error_msg)

    # Validate constraints
    constraints = get_field_constraints(field_name)
    if constraints:
        constraint_type = constraints.get("type")

        if constraint_type == "integer":
            if not isinstance(value, int):
                error_msg = f"Field '{field_name}' must be an integer"
                if description:
                    error_msg += f"\n  → Description: {description}"
                error_msg += f"\n  → You provided: {type(value).__name__} = {repr(value)}"
                return (False, error_msg)

            min_val = constraints.get("min")
            max_val = constraints.get("max")

            if min_val is not None and value < min_val:
                error_msg = f"Field '{field_name}' value {value} is below minimum {min_val}"
                if description:
                    error_msg += f"\n  → Description: {description}"
                if max_val is not None:
                    error_msg += f"\n  → Valid range: {min_val} to {max_val}"
                return (False, error_msg)

            if max_val is not None and value > max_val:
                error_msg = f"Field '{field_name}' value {value} exceeds maximum {max_val}"
                if description:
                    error_msg += f"\n  → Description: {description}"
                if min_val is not None:
                    error_msg += f"\n  → Valid range: {min_val} to {max_val}"
                return (False, error_msg)

        elif constraint_type == "string":
            if not isinstance(value, str):
                error_msg = f"Field '{field_name}' must be a string"
                if description:
                    error_msg += f"\n  → Description: {description}"
                error_msg += f"\n  → You provided: {type(value).__name__} = {repr(value)}"
                return (False, error_msg)

            max_length = constraints.get("max_length")
            if max_length and len(value) > max_length:
                error_msg = f"Field '{field_name}' length {len(value)} exceeds maximum {max_length}"
                if description:
                    error_msg += f"\n  → Description: {description}"
                error_msg += f"\n  → Your value: {repr(value[:50])}{'...' if len(value) > 50 else ''}"
                return (False, error_msg)

    return (True, None)


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "web_proxy/explicit",
    "category": "cmdb",
    "api_path": "web-proxy/explicit",
    "help": "Configure explicit Web proxy settings.",
    "total_fields": 38,
    "required_fields_count": 1,
    "fields_with_defaults_count": 36,
}


def get_schema_info() -> dict[str, Any]:
    """
    Get information about this endpoint schema.

    Returns:
        Dict with schema metadata

    Example:
        >>> info = get_schema_info()
        >>> print(f"Endpoint: {info['endpoint']}")
        >>> print(f"Total fields: {info['total_fields']}")
    """
    return SCHEMA_INFO.copy()



"""FortiOS CMDB - Firewall category"""

from .auth import Auth
from .deauth import Deauth

__all__ = [
    "Auth",
    "Deauth",
    "Firewall",
]


class Firewall:
    """Firewall endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Firewall endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.auth = Auth(client)
        self.deauth = Deauth(client)



from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class GroupPayload(TypedDict, total=False):
    """
    Type hints for application/group payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: GroupPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: NotRequired[str]  # Application group name.
    comment: NotRequired[str]  # Comments.
    type: NotRequired[Literal["application", "filter"]]  # Application group type.
    application: NotRequired[list[dict[str, Any]]]  # Application ID list.
    category: NotRequired[list[dict[str, Any]]]  # Application category ID list.
    risk: NotRequired[list[dict[str, Any]]]  # Risk, or impact, of allowing traffic from this application t
    protocols: NotRequired[str]  # Application protocol filter.
    vendor: NotRequired[str]  # Application vendor filter.
    technology: NotRequired[str]  # Application technology filter.
    behavior: NotRequired[str]  # Application behavior filter.
    popularity: NotRequired[Literal["1", "2", "3", "4", "5"]]  # Application popularity filter (1 - 5, from least to most pop


class Group:
    """
    Configure firewall application groups.
    
    Path: application/group
    Category: cmdb
    Primary Key: name
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        type: Literal["application", "filter"] | None = ...,
        application: list[dict[str, Any]] | None = ...,
        category: list[dict[str, Any]] | None = ...,
        risk: list[dict[str, Any]] | None = ...,
        protocols: str | None = ...,
        vendor: str | None = ...,
        technology: str | None = ...,
        behavior: str | None = ...,
        popularity: Literal["1", "2", "3", "4", "5"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        type: Literal["application", "filter"] | None = ...,
        application: list[dict[str, Any]] | None = ...,
        category: list[dict[str, Any]] | None = ...,
        risk: list[dict[str, Any]] | None = ...,
        protocols: str | None = ...,
        vendor: str | None = ...,
        technology: str | None = ...,
        behavior: str | None = ...,
        popularity: Literal["1", "2", "3", "4", "5"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: GroupPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "Group",
    "GroupPayload",
]


"""
FortiOS CMDB - System interface

Configuration endpoint for managing cmdb system/interface objects.

API Endpoints:
    GET    /cmdb/system/interface
    POST   /cmdb/system/interface
    PUT    /cmdb/system/interface/{identifier}
    DELETE /cmdb/system/interface/{identifier}

Example Usage:
    >>> from hfortix_fortios import FortiOS
    >>> fgt = FortiOS(host="192.168.1.99", token="your-api-token")
    >>>
    >>> # List all items
    >>> items = fgt.api.cmdb.system_interface.get()

Important:
    - Use **POST** to create new objects
    - Use **PUT** to update existing objects
    - Use **GET** to retrieve configuration
    - Use **DELETE** to remove objects
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, Union

if TYPE_CHECKING:
    from collections.abc import Coroutine
    from hfortix_core.http.interface import IHTTPClient

# Import helper functions from central _helpers module
from hfortix_fortios._helpers import (
    build_cmdb_payload,
    is_success,
)


class Interface:
    """Interface Operations."""

    def __init__(self, client: "IHTTPClient"):
        """Initialize Interface endpoint."""
        self._client = client

    def get(
        self,
        name: str | None = None,
        payload_dict: dict[str, Any] | None = None,
        vdom: str | bool | None = None,
        raw_json: bool = False,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]:
        """
        Retrieve system/interface configuration.

        Configure interfaces.

        Args:
            name: String identifier to retrieve specific object.
                If None, returns all objects.
            payload_dict: Additional query parameters (filters, format, etc.)
            vdom: Virtual domain name. Use True for global, string for specific VDOM, None for default.
            raw_json: If True, return raw API response without processing.
            **kwargs: Additional query parameters (action, format, etc.)

        Returns:
            Configuration data as dict. Returns Coroutine if using async client.
            
            Response structure:
                - http_method: GET
                - results: Configuration object(s)
                - vdom: Virtual domain
                - path: API path
                - name: Object name (single object queries)
                - status: success/error
                - http_status: HTTP status code
                - build: FortiOS build number

        Examples:
            >>> # Get all system/interface objects
            >>> result = fgt.api.cmdb.system_interface.get()
            >>> print(f"Found {len(result['results'])} objects")
            
            >>> # Get specific system/interface by name
            >>> result = fgt.api.cmdb.system_interface.get(name=1)
            >>> print(result['results'])
            
            >>> # Get with filter
            >>> result = fgt.api.cmdb.system_interface.get(
            ...     payload_dict={"filter": ["name==test"]}
            ... )
            
            >>> # Get schema information
            >>> schema = fgt.api.cmdb.system_interface.get(action="schema")

        See Also:
            - post(): Create new system/interface object
            - put(): Update existing system/interface object
            - delete(): Remove system/interface object
            - exists(): Check if object exists
        """
        params = payload_dict.copy() if payload_dict else {}
        
        if name:
            endpoint = "/system/interface/" + str(name)
        else:
            endpoint = "/system/interface"
        
        params.update(kwargs)
        return self._client.get(
            "cmdb", endpoint, params=params, vdom=vdom, raw_json=raw_json
        )


    def put(
        self,
        payload_dict: dict[str, Any] | None = None,
        name: str | None = None,
        vrf: int | None = None,
        cli_conn_status: int | None = None,
        fortilink: str | None = None,
        switch_controller_source_ip: str | None = None,
        mode: str | None = None,
        client_options: str | list | None = None,
        distance: int | None = None,
        priority: int | None = None,
        dhcp_relay_interface_select_method: str | None = None,
        dhcp_relay_interface: str | None = None,
        dhcp_relay_vrf_select: int | None = None,
        dhcp_broadcast_flag: str | None = None,
        dhcp_relay_service: str | None = None,
        dhcp_relay_ip: str | None = None,
        dhcp_relay_source_ip: str | None = None,
        dhcp_relay_circuit_id: str | None = None,
        dhcp_relay_link_selection: str | None = None,
        dhcp_relay_request_all_server: str | None = None,
        dhcp_relay_allow_no_end_option: str | None = None,
        dhcp_relay_type: str | None = None,
        dhcp_smart_relay: str | None = None,
        dhcp_relay_agent_option: str | None = None,
        dhcp_classless_route_addition: str | None = None,
        management_ip: Any | None = None,
        ip: Any | None = None,
        allowaccess: str | None = None,
        gwdetect: str | None = None,
        ping_serv_status: int | None = None,
        detectserver: str | None = None,
        detectprotocol: str | None = None,
        ha_priority: int | None = None,
        fail_detect: str | None = None,
        fail_detect_option: str | None = None,
        fail_alert_method: str | None = None,
        fail_action_on_extender: str | None = None,
        fail_alert_interfaces: str | list | None = None,
        dhcp_client_identifier: str | None = None,
        dhcp_renew_time: int | None = None,
        ipunnumbered: str | None = None,
        username: str | None = None,
        pppoe_egress_cos: str | None = None,
        pppoe_unnumbered_negotiate: str | None = None,
        password: Any | None = None,
        idle_timeout: int | None = None,
        detected_peer_mtu: int | None = None,
        disc_retry_timeout: int | None = None,
        padt_retry_timeout: int | None = None,
        service_name: str | None = None,
        ac_name: str | None = None,
        lcp_echo_interval: int | None = None,
        lcp_max_echo_fails: int | None = None,
        defaultgw: str | None = None,
        dns_server_override: str | None = None,
        dns_server_protocol: str | None = None,
        auth_type: str | None = None,
        pptp_client: str | None = None,
        pptp_user: str | None = None,
        pptp_password: Any | None = None,
        pptp_server_ip: str | None = None,
        pptp_auth_type: str | None = None,
        pptp_timeout: int | None = None,
        arpforward: str | None = None,
        ndiscforward: str | None = None,
        broadcast_forward: str | None = None,
        bfd: str | None = None,
        bfd_desired_min_tx: int | None = None,
        bfd_detect_mult: int | None = None,
        bfd_required_min_rx: int | None = None,
        l2forward: str | None = None,
        icmp_send_redirect: str | None = None,
        icmp_accept_redirect: str | None = None,
        reachable_time: int | None = None,
        vlanforward: str | None = None,
        stpforward: str | None = None,
        stpforward_mode: str | None = None,
        ips_sniffer_mode: str | None = None,
        ident_accept: str | None = None,
        ipmac: str | None = None,
        subst: str | None = None,
        macaddr: str | None = None,
        virtual_mac: str | None = None,
        substitute_dst_mac: str | None = None,
        speed: str | None = None,
        status: str | None = None,
        netbios_forward: str | None = None,
        wins_ip: str | None = None,
        type: str | None = None,
        dedicated_to: str | None = None,
        trust_ip_1: Any | None = None,
        trust_ip_2: Any | None = None,
        trust_ip_3: Any | None = None,
        trust_ip6_1: str | None = None,
        trust_ip6_2: str | None = None,
        trust_ip6_3: str | None = None,
        ring_rx: int | None = None,
        ring_tx: int | None = None,
        wccp: str | None = None,
        netflow_sampler: str | None = None,
        netflow_sample_rate: int | None = None,
        netflow_sampler_id: int | None = None,
        sflow_sampler: str | None = None,
        drop_fragment: str | None = None,
        src_check: str | None = None,
        sample_rate: int | None = None,
        polling_interval: int | None = None,
        sample_direction: str | None = None,
        explicit_web_proxy: str | None = None,
        explicit_ftp_proxy: str | None = None,
        proxy_captive_portal: str | None = None,
        tcp_mss: int | None = None,
        inbandwidth: int | None = None,
        outbandwidth: int | None = None,
        egress_shaping_profile: str | None = None,
        ingress_shaping_profile: str | None = None,
        spillover_threshold: int | None = None,
        ingress_spillover_threshold: int | None = None,
        weight: int | None = None,
        interface: str | None = None,
        external: str | None = None,
        mtu_override: str | None = None,
        mtu: int | None = None,
        vlan_protocol: str | None = None,
        vlanid: int | None = None,
        forward_domain: int | None = None,
        remote_ip: Any | None = None,
        member: str | list | None = None,
        lacp_mode: str | None = None,
        lacp_ha_secondary: str | None = None,
        system_id_type: str | None = None,
        system_id: str | None = None,
        lacp_speed: str | None = None,
        min_links: int | None = None,
        min_links_down: str | None = None,
        algorithm: str | None = None,
        link_up_delay: int | None = None,
        aggregate_type: str | None = None,
        priority_override: str | None = None,
        aggregate: str | None = None,
        redundant_interface: str | None = None,
        devindex: int | None = None,
        vindex: int | None = None,
        switch: str | None = None,
        description: str | None = None,
        alias: str | None = None,
        security_mode: str | None = None,
        security_mac_auth_bypass: str | None = None,
        security_ip_auth_bypass: str | None = None,
        security_external_web: str | None = None,
        security_external_logout: str | None = None,
        replacemsg_override_group: str | None = None,
        security_redirect_url: str | None = None,
        auth_cert: str | None = None,
        auth_portal_addr: str | None = None,
        security_exempt_list: str | None = None,
        security_groups: str | list | None = None,
        ike_saml_server: str | None = None,
        device_identification: str | None = None,
        exclude_signatures: str | None = None,
        device_user_identification: str | None = None,
        lldp_reception: str | None = None,
        lldp_transmission: str | None = None,
        lldp_network_policy: str | None = None,
        estimated_upstream_bandwidth: int | None = None,
        estimated_downstream_bandwidth: int | None = None,
        measured_upstream_bandwidth: int | None = None,
        measured_downstream_bandwidth: int | None = None,
        bandwidth_measure_time: int | None = None,
        monitor_bandwidth: str | None = None,
        vrrp_virtual_mac: str | None = None,
        vrrp: str | list | None = None,
        role: str | None = None,
        snmp_index: int | None = None,
        secondary_IP: str | None = None,
        secondaryip: str | list | None = None,
        preserve_session_route: str | None = None,
        auto_auth_extension_device: str | None = None,
        ap_discover: str | None = None,
        fortilink_neighbor_detect: str | None = None,
        ip_managed_by_fortiipam: str | None = None,
        managed_subnetwork_size: str | None = None,
        fortilink_split_interface: str | None = None,
        internal: int | None = None,
        fortilink_backup_link: int | None = None,
        switch_controller_access_vlan: str | None = None,
        switch_controller_traffic_policy: str | None = None,
        switch_controller_rspan_mode: str | None = None,
        switch_controller_netflow_collect: str | None = None,
        switch_controller_mgmt_vlan: int | None = None,
        switch_controller_igmp_snooping: str | None = None,
        switch_controller_igmp_snooping_proxy: str | None = None,
        switch_controller_igmp_snooping_fast_leave: str | None = None,
        switch_controller_dhcp_snooping: str | None = None,
        switch_controller_dhcp_snooping_verify_mac: str | None = None,
        switch_controller_dhcp_snooping_option82: str | None = None,
        dhcp_snooping_server_list: str | list | None = None,
        switch_controller_arp_inspection: str | None = None,
        switch_controller_learning_limit: int | None = None,
        switch_controller_nac: str | None = None,
        switch_controller_dynamic: str | None = None,
        switch_controller_feature: str | None = None,
        switch_controller_iot_scanning: str | None = None,
        switch_controller_offload: str | None = None,
        switch_controller_offload_ip: str | None = None,
        switch_controller_offload_gw: str | None = None,
        swc_vlan: int | None = None,
        swc_first_create: int | None = None,
        color: int | None = None,
        tagging: str | list | None = None,
        eap_supplicant: str | None = None,
        eap_method: str | None = None,
        eap_identity: str | None = None,
        eap_password: Any | None = None,
        eap_ca_cert: str | None = None,
        eap_user_cert: str | None = None,
        default_purdue_level: str | None = None,
        ipv6: str | None = None,
        physical: Any | None = None,
        vdom: str | bool | None = None,
        raw_json: bool = False,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]:
        """
        Update existing system/interface object.

        Configure interfaces.

        Args:
            payload_dict: Object data as dict. Must include name (primary key).
            name: Name.
            vdom: Interface is in this virtual domain (VDOM).
            vrf: Virtual Routing Forwarding ID.
            cli_conn_status: CLI connection status.
            fortilink: Enable FortiLink to dedicate this interface to manage other Fortinet devices.
            vdom: Virtual domain name.
            raw_json: If True, return raw API response.
            **kwargs: Additional parameters

        Returns:
            API response dict

        Raises:
            ValueError: If name is missing from payload

        Examples:
            >>> # Update specific fields
            >>> result = fgt.api.cmdb.system_interface.put(
            ...     name=1,
            ...     # ... fields to update
            ... )
            
            >>> # Update using payload dict
            >>> payload = {
            ...     "name": 1,
            ...     "field1": "new-value",
            ... }
            >>> result = fgt.api.cmdb.system_interface.put(payload_dict=payload)

        See Also:
            - post(): Create new object
            - set(): Intelligent create or update
        """
        # Build payload using helper function
        # Note: Skip reserved parameters (data, vdom, raw_json, kwargs) and Python keywords from field list
        payload_data = build_cmdb_payload(
            name=name,
            vrf=vrf,
            cli_conn_status=cli_conn_status,
            fortilink=fortilink,
            switch_controller_source_ip=switch_controller_source_ip,
            mode=mode,
            client_options=client_options,
            distance=distance,
            priority=priority,
            dhcp_relay_interface_select_method=dhcp_relay_interface_select_method,
            dhcp_relay_interface=dhcp_relay_interface,
            dhcp_relay_vrf_select=dhcp_relay_vrf_select,
            dhcp_broadcast_flag=dhcp_broadcast_flag,
            dhcp_relay_service=dhcp_relay_service,
            dhcp_relay_ip=dhcp_relay_ip,
            dhcp_relay_source_ip=dhcp_relay_source_ip,
            dhcp_relay_circuit_id=dhcp_relay_circuit_id,
            dhcp_relay_link_selection=dhcp_relay_link_selection,
            dhcp_relay_request_all_server=dhcp_relay_request_all_server,
            dhcp_relay_allow_no_end_option=dhcp_relay_allow_no_end_option,
            dhcp_relay_type=dhcp_relay_type,
            dhcp_smart_relay=dhcp_smart_relay,
            dhcp_relay_agent_option=dhcp_relay_agent_option,
            dhcp_classless_route_addition=dhcp_classless_route_addition,
            management_ip=management_ip,
            ip=ip,
            allowaccess=allowaccess,
            gwdetect=gwdetect,
            ping_serv_status=ping_serv_status,
            detectserver=detectserver,
            detectprotocol=detectprotocol,
            ha_priority=ha_priority,
            fail_detect=fail_detect,
            fail_detect_option=fail_detect_option,
            fail_alert_method=fail_alert_method,
            fail_action_on_extender=fail_action_on_extender,
            fail_alert_interfaces=fail_alert_interfaces,
            dhcp_client_identifier=dhcp_client_identifier,
            dhcp_renew_time=dhcp_renew_time,
            ipunnumbered=ipunnumbered,
            username=username,
            pppoe_egress_cos=pppoe_egress_cos,
            pppoe_unnumbered_negotiate=pppoe_unnumbered_negotiate,
            password=password,
            idle_timeout=idle_timeout,
            detected_peer_mtu=detected_peer_mtu,
            disc_retry_timeout=disc_retry_timeout,
            padt_retry_timeout=padt_retry_timeout,
            service_name=service_name,
            ac_name=ac_name,
            lcp_echo_interval=lcp_echo_interval,
            lcp_max_echo_fails=lcp_max_echo_fails,
            defaultgw=defaultgw,
            dns_server_override=dns_server_override,
            dns_server_protocol=dns_server_protocol,
            auth_type=auth_type,
            pptp_client=pptp_client,
            pptp_user=pptp_user,
            pptp_password=pptp_password,
            pptp_server_ip=pptp_server_ip,
            pptp_auth_type=pptp_auth_type,
            pptp_timeout=pptp_timeout,
            arpforward=arpforward,
            ndiscforward=ndiscforward,
            broadcast_forward=broadcast_forward,
            bfd=bfd,
            bfd_desired_min_tx=bfd_desired_min_tx,
            bfd_detect_mult=bfd_detect_mult,
            bfd_required_min_rx=bfd_required_min_rx,
            l2forward=l2forward,
            icmp_send_redirect=icmp_send_redirect,
            icmp_accept_redirect=icmp_accept_redirect,
            reachable_time=reachable_time,
            vlanforward=vlanforward,
            stpforward=stpforward,
            stpforward_mode=stpforward_mode,
            ips_sniffer_mode=ips_sniffer_mode,
            ident_accept=ident_accept,
            ipmac=ipmac,
            subst=subst,
            macaddr=macaddr,
            virtual_mac=virtual_mac,
            substitute_dst_mac=substitute_dst_mac,
            speed=speed,
            status=status,
            netbios_forward=netbios_forward,
            wins_ip=wins_ip,
            type=type,
            dedicated_to=dedicated_to,
            trust_ip_1=trust_ip_1,
            trust_ip_2=trust_ip_2,
            trust_ip_3=trust_ip_3,
            trust_ip6_1=trust_ip6_1,
            trust_ip6_2=trust_ip6_2,
            trust_ip6_3=trust_ip6_3,
            ring_rx=ring_rx,
            ring_tx=ring_tx,
            wccp=wccp,
            netflow_sampler=netflow_sampler,
            netflow_sample_rate=netflow_sample_rate,
            netflow_sampler_id=netflow_sampler_id,
            sflow_sampler=sflow_sampler,
            drop_fragment=drop_fragment,
            src_check=src_check,
            sample_rate=sample_rate,
            polling_interval=polling_interval,
            sample_direction=sample_direction,
            explicit_web_proxy=explicit_web_proxy,
            explicit_ftp_proxy=explicit_ftp_proxy,
            proxy_captive_portal=proxy_captive_portal,
            tcp_mss=tcp_mss,
            inbandwidth=inbandwidth,
            outbandwidth=outbandwidth,
            egress_shaping_profile=egress_shaping_profile,
            ingress_shaping_profile=ingress_shaping_profile,
            spillover_threshold=spillover_threshold,
            ingress_spillover_threshold=ingress_spillover_threshold,
            weight=weight,
            interface=interface,
            external=external,
            mtu_override=mtu_override,
            mtu=mtu,
            vlan_protocol=vlan_protocol,
            vlanid=vlanid,
            forward_domain=forward_domain,
            remote_ip=remote_ip,
            member=member,
            lacp_mode=lacp_mode,
            lacp_ha_secondary=lacp_ha_secondary,
            system_id_type=system_id_type,
            system_id=system_id,
            lacp_speed=lacp_speed,
            min_links=min_links,
            min_links_down=min_links_down,
            algorithm=algorithm,
            link_up_delay=link_up_delay,
            aggregate_type=aggregate_type,
            priority_override=priority_override,
            aggregate=aggregate,
            redundant_interface=redundant_interface,
            devindex=devindex,
            vindex=vindex,
            switch=switch,
            description=description,
            alias=alias,
            security_mode=security_mode,
            security_mac_auth_bypass=security_mac_auth_bypass,
            security_ip_auth_bypass=security_ip_auth_bypass,
            security_external_web=security_external_web,
            security_external_logout=security_external_logout,
            replacemsg_override_group=replacemsg_override_group,
            security_redirect_url=security_redirect_url,
            auth_cert=auth_cert,
            auth_portal_addr=auth_portal_addr,
            security_exempt_list=security_exempt_list,
            security_groups=security_groups,
            ike_saml_server=ike_saml_server,
            device_identification=device_identification,
            exclude_signatures=exclude_signatures,
            device_user_identification=device_user_identification,
            lldp_reception=lldp_reception,
            lldp_transmission=lldp_transmission,
            lldp_network_policy=lldp_network_policy,
            estimated_upstream_bandwidth=estimated_upstream_bandwidth,
            estimated_downstream_bandwidth=estimated_downstream_bandwidth,
            measured_upstream_bandwidth=measured_upstream_bandwidth,
            measured_downstream_bandwidth=measured_downstream_bandwidth,
            bandwidth_measure_time=bandwidth_measure_time,
            monitor_bandwidth=monitor_bandwidth,
            vrrp_virtual_mac=vrrp_virtual_mac,
            vrrp=vrrp,
            role=role,
            snmp_index=snmp_index,
            secondary_IP=secondary_IP,
            secondaryip=secondaryip,
            preserve_session_route=preserve_session_route,
            auto_auth_extension_device=auto_auth_extension_device,
            ap_discover=ap_discover,
            fortilink_neighbor_detect=fortilink_neighbor_detect,
            ip_managed_by_fortiipam=ip_managed_by_fortiipam,
            managed_subnetwork_size=managed_subnetwork_size,
            fortilink_split_interface=fortilink_split_interface,
            internal=internal,
            fortilink_backup_link=fortilink_backup_link,
            switch_controller_access_vlan=switch_controller_access_vlan,
            switch_controller_traffic_policy=switch_controller_traffic_policy,
            switch_controller_rspan_mode=switch_controller_rspan_mode,
            switch_controller_netflow_collect=switch_controller_netflow_collect,
            switch_controller_mgmt_vlan=switch_controller_mgmt_vlan,
            switch_controller_igmp_snooping=switch_controller_igmp_snooping,
            switch_controller_igmp_snooping_proxy=switch_controller_igmp_snooping_proxy,
            switch_controller_igmp_snooping_fast_leave=switch_controller_igmp_snooping_fast_leave,
            switch_controller_dhcp_snooping=switch_controller_dhcp_snooping,
            switch_controller_dhcp_snooping_verify_mac=switch_controller_dhcp_snooping_verify_mac,
            switch_controller_dhcp_snooping_option82=switch_controller_dhcp_snooping_option82,
            dhcp_snooping_server_list=dhcp_snooping_server_list,
            switch_controller_arp_inspection=switch_controller_arp_inspection,
            switch_controller_learning_limit=switch_controller_learning_limit,
            switch_controller_nac=switch_controller_nac,
            switch_controller_dynamic=switch_controller_dynamic,
            switch_controller_feature=switch_controller_feature,
            switch_controller_iot_scanning=switch_controller_iot_scanning,
            switch_controller_offload=switch_controller_offload,
            switch_controller_offload_ip=switch_controller_offload_ip,
            switch_controller_offload_gw=switch_controller_offload_gw,
            swc_vlan=swc_vlan,
            swc_first_create=swc_first_create,
            color=color,
            tagging=tagging,
            eap_supplicant=eap_supplicant,
            eap_method=eap_method,
            eap_identity=eap_identity,
            eap_password=eap_password,
            eap_ca_cert=eap_ca_cert,
            eap_user_cert=eap_user_cert,
            default_purdue_level=default_purdue_level,
            ipv6=ipv6,
            physical=physical,
            data=payload_dict,
        )
        
        # Check for deprecated fields and warn users
        from ._helpers.interface import DEPRECATED_FIELDS
        if DEPRECATED_FIELDS:
            from hfortix_core import check_deprecated_fields
            check_deprecated_fields(
                payload=payload_data,
                deprecated_fields=DEPRECATED_FIELDS,
                endpoint="cmdb/system/interface",
            )
        
        name_value = payload_data.get("name")
        if not name_value:
            raise ValueError("name is required for PUT")
        endpoint = "/system/interface/" + str(name_value)

        return self._client.put(
            "cmdb", endpoint, data=payload_data, params=kwargs, vdom=vdom, raw_json=raw_json
        )

    def post(
        self,
        payload_dict: dict[str, Any] | None = None,
        name: str | None = None,
        vrf: int | None = None,
        cli_conn_status: int | None = None,
        fortilink: str | None = None,
        switch_controller_source_ip: str | None = None,
        mode: str | None = None,
        client_options: str | list | None = None,
        distance: int | None = None,
        priority: int | None = None,
        dhcp_relay_interface_select_method: str | None = None,
        dhcp_relay_interface: str | None = None,
        dhcp_relay_vrf_select: int | None = None,
        dhcp_broadcast_flag: str | None = None,
        dhcp_relay_service: str | None = None,
        dhcp_relay_ip: str | None = None,
        dhcp_relay_source_ip: str | None = None,
        dhcp_relay_circuit_id: str | None = None,
        dhcp_relay_link_selection: str | None = None,
        dhcp_relay_request_all_server: str | None = None,
        dhcp_relay_allow_no_end_option: str | None = None,
        dhcp_relay_type: str | None = None,
        dhcp_smart_relay: str | None = None,
        dhcp_relay_agent_option: str | None = None,
        dhcp_classless_route_addition: str | None = None,
        management_ip: Any | None = None,
        ip: Any | None = None,
        allowaccess: str | None = None,
        gwdetect: str | None = None,
        ping_serv_status: int | None = None,
        detectserver: str | None = None,
        detectprotocol: str | None = None,
        ha_priority: int | None = None,
        fail_detect: str | None = None,
        fail_detect_option: str | None = None,
        fail_alert_method: str | None = None,
        fail_action_on_extender: str | None = None,
        fail_alert_interfaces: str | list | None = None,
        dhcp_client_identifier: str | None = None,
        dhcp_renew_time: int | None = None,
        ipunnumbered: str | None = None,
        username: str | None = None,
        pppoe_egress_cos: str | None = None,
        pppoe_unnumbered_negotiate: str | None = None,
        password: Any | None = None,
        idle_timeout: int | None = None,
        detected_peer_mtu: int | None = None,
        disc_retry_timeout: int | None = None,
        padt_retry_timeout: int | None = None,
        service_name: str | None = None,
        ac_name: str | None = None,
        lcp_echo_interval: int | None = None,
        lcp_max_echo_fails: int | None = None,
        defaultgw: str | None = None,
        dns_server_override: str | None = None,
        dns_server_protocol: str | None = None,
        auth_type: str | None = None,
        pptp_client: str | None = None,
        pptp_user: str | None = None,
        pptp_password: Any | None = None,
        pptp_server_ip: str | None = None,
        pptp_auth_type: str | None = None,
        pptp_timeout: int | None = None,
        arpforward: str | None = None,
        ndiscforward: str | None = None,
        broadcast_forward: str | None = None,
        bfd: str | None = None,
        bfd_desired_min_tx: int | None = None,
        bfd_detect_mult: int | None = None,
        bfd_required_min_rx: int | None = None,
        l2forward: str | None = None,
        icmp_send_redirect: str | None = None,
        icmp_accept_redirect: str | None = None,
        reachable_time: int | None = None,
        vlanforward: str | None = None,
        stpforward: str | None = None,
        stpforward_mode: str | None = None,
        ips_sniffer_mode: str | None = None,
        ident_accept: str | None = None,
        ipmac: str | None = None,
        subst: str | None = None,
        macaddr: str | None = None,
        virtual_mac: str | None = None,
        substitute_dst_mac: str | None = None,
        speed: str | None = None,
        status: str | None = None,
        netbios_forward: str | None = None,
        wins_ip: str | None = None,
        type: str | None = None,
        dedicated_to: str | None = None,
        trust_ip_1: Any | None = None,
        trust_ip_2: Any | None = None,
        trust_ip_3: Any | None = None,
        trust_ip6_1: str | None = None,
        trust_ip6_2: str | None = None,
        trust_ip6_3: str | None = None,
        ring_rx: int | None = None,
        ring_tx: int | None = None,
        wccp: str | None = None,
        netflow_sampler: str | None = None,
        netflow_sample_rate: int | None = None,
        netflow_sampler_id: int | None = None,
        sflow_sampler: str | None = None,
        drop_fragment: str | None = None,
        src_check: str | None = None,
        sample_rate: int | None = None,
        polling_interval: int | None = None,
        sample_direction: str | None = None,
        explicit_web_proxy: str | None = None,
        explicit_ftp_proxy: str | None = None,
        proxy_captive_portal: str | None = None,
        tcp_mss: int | None = None,
        inbandwidth: int | None = None,
        outbandwidth: int | None = None,
        egress_shaping_profile: str | None = None,
        ingress_shaping_profile: str | None = None,
        spillover_threshold: int | None = None,
        ingress_spillover_threshold: int | None = None,
        weight: int | None = None,
        interface: str | None = None,
        external: str | None = None,
        mtu_override: str | None = None,
        mtu: int | None = None,
        vlan_protocol: str | None = None,
        vlanid: int | None = None,
        forward_domain: int | None = None,
        remote_ip: Any | None = None,
        member: str | list | None = None,
        lacp_mode: str | None = None,
        lacp_ha_secondary: str | None = None,
        system_id_type: str | None = None,
        system_id: str | None = None,
        lacp_speed: str | None = None,
        min_links: int | None = None,
        min_links_down: str | None = None,
        algorithm: str | None = None,
        link_up_delay: int | None = None,
        aggregate_type: str | None = None,
        priority_override: str | None = None,
        aggregate: str | None = None,
        redundant_interface: str | None = None,
        devindex: int | None = None,
        vindex: int | None = None,
        switch: str | None = None,
        description: str | None = None,
        alias: str | None = None,
        security_mode: str | None = None,
        security_mac_auth_bypass: str | None = None,
        security_ip_auth_bypass: str | None = None,
        security_external_web: str | None = None,
        security_external_logout: str | None = None,
        replacemsg_override_group: str | None = None,
        security_redirect_url: str | None = None,
        auth_cert: str | None = None,
        auth_portal_addr: str | None = None,
        security_exempt_list: str | None = None,
        security_groups: str | list | None = None,
        ike_saml_server: str | None = None,
        device_identification: str | None = None,
        exclude_signatures: str | None = None,
        device_user_identification: str | None = None,
        lldp_reception: str | None = None,
        lldp_transmission: str | None = None,
        lldp_network_policy: str | None = None,
        estimated_upstream_bandwidth: int | None = None,
        estimated_downstream_bandwidth: int | None = None,
        measured_upstream_bandwidth: int | None = None,
        measured_downstream_bandwidth: int | None = None,
        bandwidth_measure_time: int | None = None,
        monitor_bandwidth: str | None = None,
        vrrp_virtual_mac: str | None = None,
        vrrp: str | list | None = None,
        role: str | None = None,
        snmp_index: int | None = None,
        secondary_IP: str | None = None,
        secondaryip: str | list | None = None,
        preserve_session_route: str | None = None,
        auto_auth_extension_device: str | None = None,
        ap_discover: str | None = None,
        fortilink_neighbor_detect: str | None = None,
        ip_managed_by_fortiipam: str | None = None,
        managed_subnetwork_size: str | None = None,
        fortilink_split_interface: str | None = None,
        internal: int | None = None,
        fortilink_backup_link: int | None = None,
        switch_controller_access_vlan: str | None = None,
        switch_controller_traffic_policy: str | None = None,
        switch_controller_rspan_mode: str | None = None,
        switch_controller_netflow_collect: str | None = None,
        switch_controller_mgmt_vlan: int | None = None,
        switch_controller_igmp_snooping: str | None = None,
        switch_controller_igmp_snooping_proxy: str | None = None,
        switch_controller_igmp_snooping_fast_leave: str | None = None,
        switch_controller_dhcp_snooping: str | None = None,
        switch_controller_dhcp_snooping_verify_mac: str | None = None,
        switch_controller_dhcp_snooping_option82: str | None = None,
        dhcp_snooping_server_list: str | list | None = None,
        switch_controller_arp_inspection: str | None = None,
        switch_controller_learning_limit: int | None = None,
        switch_controller_nac: str | None = None,
        switch_controller_dynamic: str | None = None,
        switch_controller_feature: str | None = None,
        switch_controller_iot_scanning: str | None = None,
        switch_controller_offload: str | None = None,
        switch_controller_offload_ip: str | None = None,
        switch_controller_offload_gw: str | None = None,
        swc_vlan: int | None = None,
        swc_first_create: int | None = None,
        color: int | None = None,
        tagging: str | list | None = None,
        eap_supplicant: str | None = None,
        eap_method: str | None = None,
        eap_identity: str | None = None,
        eap_password: Any | None = None,
        eap_ca_cert: str | None = None,
        eap_user_cert: str | None = None,
        default_purdue_level: str | None = None,
        ipv6: str | None = None,
        physical: Any | None = None,
        vdom: str | bool | None = None,
        raw_json: bool = False,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]:
        """
        Create new system/interface object.

        Configure interfaces.

        Args:
            payload_dict: Complete object data as dict. Alternative to individual parameters.
            name: Name.
            vdom: Interface is in this virtual domain (VDOM).
            vrf: Virtual Routing Forwarding ID.
            cli_conn_status: CLI connection status.
            fortilink: Enable FortiLink to dedicate this interface to manage other Fortinet devices.
            vdom: Virtual domain name. Use True for global, string for specific VDOM.
            raw_json: If True, return raw API response without processing.
            **kwargs: Additional parameters

        Returns:
            API response dict containing created object with assigned name.

        Examples:
            >>> # Create using individual parameters
            >>> result = fgt.api.cmdb.system_interface.post(
            ...     name="example",
            ...     # ... other required fields
            ... )
            >>> print(f"Created name: {result['results']}")
            
            >>> # Create using payload dict
            >>> payload = Interface.defaults()  # Start with defaults
            >>> payload['name'] = 'my-object'
            >>> result = fgt.api.cmdb.system_interface.post(payload_dict=payload)

        Note:
            Required fields: {{ ", ".join(Interface.required_fields()) }}
            
            Use Interface.help('field_name') to get field details.

        See Also:
            - get(): Retrieve objects
            - put(): Update existing object
            - set(): Intelligent create or update
        """
        # Build payload using helper function
        # Note: Skip reserved parameters (data, vdom, raw_json, kwargs) and Python keywords from field list
        payload_data = build_cmdb_payload(
            name=name,
            vrf=vrf,
            cli_conn_status=cli_conn_status,
            fortilink=fortilink,
            switch_controller_source_ip=switch_controller_source_ip,
            mode=mode,
            client_options=client_options,
            distance=distance,
            priority=priority,
            dhcp_relay_interface_select_method=dhcp_relay_interface_select_method,
            dhcp_relay_interface=dhcp_relay_interface,
            dhcp_relay_vrf_select=dhcp_relay_vrf_select,
            dhcp_broadcast_flag=dhcp_broadcast_flag,
            dhcp_relay_service=dhcp_relay_service,
            dhcp_relay_ip=dhcp_relay_ip,
            dhcp_relay_source_ip=dhcp_relay_source_ip,
            dhcp_relay_circuit_id=dhcp_relay_circuit_id,
            dhcp_relay_link_selection=dhcp_relay_link_selection,
            dhcp_relay_request_all_server=dhcp_relay_request_all_server,
            dhcp_relay_allow_no_end_option=dhcp_relay_allow_no_end_option,
            dhcp_relay_type=dhcp_relay_type,
            dhcp_smart_relay=dhcp_smart_relay,
            dhcp_relay_agent_option=dhcp_relay_agent_option,
            dhcp_classless_route_addition=dhcp_classless_route_addition,
            management_ip=management_ip,
            ip=ip,
            allowaccess=allowaccess,
            gwdetect=gwdetect,
            ping_serv_status=ping_serv_status,
            detectserver=detectserver,
            detectprotocol=detectprotocol,
            ha_priority=ha_priority,
            fail_detect=fail_detect,
            fail_detect_option=fail_detect_option,
            fail_alert_method=fail_alert_method,
            fail_action_on_extender=fail_action_on_extender,
            fail_alert_interfaces=fail_alert_interfaces,
            dhcp_client_identifier=dhcp_client_identifier,
            dhcp_renew_time=dhcp_renew_time,
            ipunnumbered=ipunnumbered,
            username=username,
            pppoe_egress_cos=pppoe_egress_cos,
            pppoe_unnumbered_negotiate=pppoe_unnumbered_negotiate,
            password=password,
            idle_timeout=idle_timeout,
            detected_peer_mtu=detected_peer_mtu,
            disc_retry_timeout=disc_retry_timeout,
            padt_retry_timeout=padt_retry_timeout,
            service_name=service_name,
            ac_name=ac_name,
            lcp_echo_interval=lcp_echo_interval,
            lcp_max_echo_fails=lcp_max_echo_fails,
            defaultgw=defaultgw,
            dns_server_override=dns_server_override,
            dns_server_protocol=dns_server_protocol,
            auth_type=auth_type,
            pptp_client=pptp_client,
            pptp_user=pptp_user,
            pptp_password=pptp_password,
            pptp_server_ip=pptp_server_ip,
            pptp_auth_type=pptp_auth_type,
            pptp_timeout=pptp_timeout,
            arpforward=arpforward,
            ndiscforward=ndiscforward,
            broadcast_forward=broadcast_forward,
            bfd=bfd,
            bfd_desired_min_tx=bfd_desired_min_tx,
            bfd_detect_mult=bfd_detect_mult,
            bfd_required_min_rx=bfd_required_min_rx,
            l2forward=l2forward,
            icmp_send_redirect=icmp_send_redirect,
            icmp_accept_redirect=icmp_accept_redirect,
            reachable_time=reachable_time,
            vlanforward=vlanforward,
            stpforward=stpforward,
            stpforward_mode=stpforward_mode,
            ips_sniffer_mode=ips_sniffer_mode,
            ident_accept=ident_accept,
            ipmac=ipmac,
            subst=subst,
            macaddr=macaddr,
            virtual_mac=virtual_mac,
            substitute_dst_mac=substitute_dst_mac,
            speed=speed,
            status=status,
            netbios_forward=netbios_forward,
            wins_ip=wins_ip,
            type=type,
            dedicated_to=dedicated_to,
            trust_ip_1=trust_ip_1,
            trust_ip_2=trust_ip_2,
            trust_ip_3=trust_ip_3,
            trust_ip6_1=trust_ip6_1,
            trust_ip6_2=trust_ip6_2,
            trust_ip6_3=trust_ip6_3,
            ring_rx=ring_rx,
            ring_tx=ring_tx,
            wccp=wccp,
            netflow_sampler=netflow_sampler,
            netflow_sample_rate=netflow_sample_rate,
            netflow_sampler_id=netflow_sampler_id,
            sflow_sampler=sflow_sampler,
            drop_fragment=drop_fragment,
            src_check=src_check,
            sample_rate=sample_rate,
            polling_interval=polling_interval,
            sample_direction=sample_direction,
            explicit_web_proxy=explicit_web_proxy,
            explicit_ftp_proxy=explicit_ftp_proxy,
            proxy_captive_portal=proxy_captive_portal,
            tcp_mss=tcp_mss,
            inbandwidth=inbandwidth,
            outbandwidth=outbandwidth,
            egress_shaping_profile=egress_shaping_profile,
            ingress_shaping_profile=ingress_shaping_profile,
            spillover_threshold=spillover_threshold,
            ingress_spillover_threshold=ingress_spillover_threshold,
            weight=weight,
            interface=interface,
            external=external,
            mtu_override=mtu_override,
            mtu=mtu,
            vlan_protocol=vlan_protocol,
            vlanid=vlanid,
            forward_domain=forward_domain,
            remote_ip=remote_ip,
            member=member,
            lacp_mode=lacp_mode,
            lacp_ha_secondary=lacp_ha_secondary,
            system_id_type=system_id_type,
            system_id=system_id,
            lacp_speed=lacp_speed,
            min_links=min_links,
            min_links_down=min_links_down,
            algorithm=algorithm,
            link_up_delay=link_up_delay,
            aggregate_type=aggregate_type,
            priority_override=priority_override,
            aggregate=aggregate,
            redundant_interface=redundant_interface,
            devindex=devindex,
            vindex=vindex,
            switch=switch,
            description=description,
            alias=alias,
            security_mode=security_mode,
            security_mac_auth_bypass=security_mac_auth_bypass,
            security_ip_auth_bypass=security_ip_auth_bypass,
            security_external_web=security_external_web,
            security_external_logout=security_external_logout,
            replacemsg_override_group=replacemsg_override_group,
            security_redirect_url=security_redirect_url,
            auth_cert=auth_cert,
            auth_portal_addr=auth_portal_addr,
            security_exempt_list=security_exempt_list,
            security_groups=security_groups,
            ike_saml_server=ike_saml_server,
            device_identification=device_identification,
            exclude_signatures=exclude_signatures,
            device_user_identification=device_user_identification,
            lldp_reception=lldp_reception,
            lldp_transmission=lldp_transmission,
            lldp_network_policy=lldp_network_policy,
            estimated_upstream_bandwidth=estimated_upstream_bandwidth,
            estimated_downstream_bandwidth=estimated_downstream_bandwidth,
            measured_upstream_bandwidth=measured_upstream_bandwidth,
            measured_downstream_bandwidth=measured_downstream_bandwidth,
            bandwidth_measure_time=bandwidth_measure_time,
            monitor_bandwidth=monitor_bandwidth,
            vrrp_virtual_mac=vrrp_virtual_mac,
            vrrp=vrrp,
            role=role,
            snmp_index=snmp_index,
            secondary_IP=secondary_IP,
            secondaryip=secondaryip,
            preserve_session_route=preserve_session_route,
            auto_auth_extension_device=auto_auth_extension_device,
            ap_discover=ap_discover,
            fortilink_neighbor_detect=fortilink_neighbor_detect,
            ip_managed_by_fortiipam=ip_managed_by_fortiipam,
            managed_subnetwork_size=managed_subnetwork_size,
            fortilink_split_interface=fortilink_split_interface,
            internal=internal,
            fortilink_backup_link=fortilink_backup_link,
            switch_controller_access_vlan=switch_controller_access_vlan,
            switch_controller_traffic_policy=switch_controller_traffic_policy,
            switch_controller_rspan_mode=switch_controller_rspan_mode,
            switch_controller_netflow_collect=switch_controller_netflow_collect,
            switch_controller_mgmt_vlan=switch_controller_mgmt_vlan,
            switch_controller_igmp_snooping=switch_controller_igmp_snooping,
            switch_controller_igmp_snooping_proxy=switch_controller_igmp_snooping_proxy,
            switch_controller_igmp_snooping_fast_leave=switch_controller_igmp_snooping_fast_leave,
            switch_controller_dhcp_snooping=switch_controller_dhcp_snooping,
            switch_controller_dhcp_snooping_verify_mac=switch_controller_dhcp_snooping_verify_mac,
            switch_controller_dhcp_snooping_option82=switch_controller_dhcp_snooping_option82,
            dhcp_snooping_server_list=dhcp_snooping_server_list,
            switch_controller_arp_inspection=switch_controller_arp_inspection,
            switch_controller_learning_limit=switch_controller_learning_limit,
            switch_controller_nac=switch_controller_nac,
            switch_controller_dynamic=switch_controller_dynamic,
            switch_controller_feature=switch_controller_feature,
            switch_controller_iot_scanning=switch_controller_iot_scanning,
            switch_controller_offload=switch_controller_offload,
            switch_controller_offload_ip=switch_controller_offload_ip,
            switch_controller_offload_gw=switch_controller_offload_gw,
            swc_vlan=swc_vlan,
            swc_first_create=swc_first_create,
            color=color,
            tagging=tagging,
            eap_supplicant=eap_supplicant,
            eap_method=eap_method,
            eap_identity=eap_identity,
            eap_password=eap_password,
            eap_ca_cert=eap_ca_cert,
            eap_user_cert=eap_user_cert,
            default_purdue_level=default_purdue_level,
            ipv6=ipv6,
            physical=physical,
            data=payload_dict,
        )

        # Check for deprecated fields and warn users
        from ._helpers.interface import DEPRECATED_FIELDS
        if DEPRECATED_FIELDS:
            from hfortix_core import check_deprecated_fields
            check_deprecated_fields(
                payload=payload_data,
                deprecated_fields=DEPRECATED_FIELDS,
                endpoint="cmdb/system/interface",
            )

        endpoint = "/system/interface"
        return self._client.post(
            "cmdb", endpoint, data=payload_data, params=kwargs, vdom=vdom, raw_json=raw_json
        )

    def delete(
        self,
        name: str | None = None,
        vdom: str | bool | None = None,
        raw_json: bool = False,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]:
        """
        Delete system/interface object.

        Configure interfaces.

        Args:
            name: Primary key identifier
            vdom: Virtual domain name
            raw_json: If True, return raw API response
            **kwargs: Additional parameters

        Returns:
            API response dict

        Raises:
            ValueError: If name is not provided

        Examples:
            >>> # Delete specific object
            >>> result = fgt.api.cmdb.system_interface.delete(name=1)
            
            >>> # Check for errors
            >>> if result.get('status') != 'success':
            ...     print(f"Delete failed: {result.get('error')}")

        See Also:
            - exists(): Check if object exists before deleting
            - get(): Retrieve object to verify it exists
        """
        if not name:
            raise ValueError("name is required for DELETE")
        endpoint = "/system/interface/" + str(name)

        return self._client.delete(
            "cmdb", endpoint, params=kwargs, vdom=vdom, raw_json=raw_json
        )

    def exists(
        self,
        name: str,
        vdom: str | bool | None = None,
    ) -> Union[bool, Coroutine[Any, Any, bool]]:
        """
        Check if system/interface object exists.

        Verifies whether an object exists by attempting to retrieve it and checking the response status.

        Args:
            name: Primary key identifier
            vdom: Virtual domain name

        Returns:
            True if object exists, False otherwise

        Examples:
            >>> # Check if object exists before operations
            >>> if fgt.api.cmdb.system_interface.exists(name=1):
            ...     print("Object exists")
            ... else:
            ...     print("Object not found")
            
            >>> # Conditional delete
            >>> if fgt.api.cmdb.system_interface.exists(name=1):
            ...     fgt.api.cmdb.system_interface.delete(name=1)

        See Also:
            - get(): Retrieve full object data
            - set(): Create or update automatically based on existence
        """
        try:
            response = self.get(name=name, vdom=vdom, raw_json=True)
            
            if isinstance(response, dict):
                # Use helper function to check success
                return is_success(response)
            else:
                async def _check() -> bool:
                    r = await response
                    return is_success(r)
                return _check()
        except Exception:
            # Resource not found or other error - return False
            return False


    def set(
        self,
        payload_dict: dict[str, Any] | None = None,
        vdom: str | bool | None = None,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]:
        """
        Create or update system/interface object (intelligent operation).

        Automatically determines whether to create (POST) or update (PUT) based on
        whether the resource exists. Requires the primary key (name) in the payload.

        Args:
            payload_dict: Resource data including name (primary key)
            vdom: Virtual domain name
            **kwargs: Additional parameters passed to PUT or POST

        Returns:
            API response dictionary

        Raises:
            ValueError: If name is missing from payload

        Examples:
            >>> # Intelligent create or update - no need to check exists()
            >>> payload = {
            ...     "name": 1,
            ...     "field1": "value1",
            ...     "field2": "value2",
            ... }
            >>> result = fgt.api.cmdb.system_interface.set(payload_dict=payload)
            >>> # Will POST if object doesn't exist, PUT if it does
            
            >>> # Idempotent configuration
            >>> for obj_data in configuration_list:
            ...     fgt.api.cmdb.system_interface.set(payload_dict=obj_data)
            >>> # Safely applies configuration regardless of current state

        Note:
            This method internally calls exists() then either post() or put().
            For performance-critical code with known state, call post() or put() directly.

        See Also:
            - post(): Create new object
            - put(): Update existing object
            - exists(): Check existence manually
        """
        if payload_dict is None:
            payload_dict = {}
        
        mkey_value = payload_dict.get("name")
        if not mkey_value:
            raise ValueError("name is required in payload_dict for set()")
        
        # Check if resource exists
        if self.exists(name=mkey_value, vdom=vdom):
            # Update existing resource
            return self.put(payload_dict=payload_dict, vdom=vdom, **kwargs)
        else:
            # Create new resource
            return self.post(payload_dict=payload_dict, vdom=vdom, **kwargs)


    # ========================================================================
    # Metadata Helper Methods
    # Provide easy access to schema metadata without separate imports
    # ========================================================================

    @staticmethod
    def help(field_name: str | None = None) -> str:
        """
        Get help text for endpoint or specific field.

        Args:
            field_name: Optional field name to get help for. If None, shows endpoint help.

        Returns:
            Formatted help text

        Examples:
            >>> # Get endpoint information
            >>> print(Interface.help())
            
            >>> # Get field information
            >>> print(Interface.help("name"))
        """
        from ._helpers.interface import (
            get_schema_info,
            get_field_metadata,
        )

        if field_name is None:
            # Endpoint help
            info = get_schema_info()
            lines = [
                f"Endpoint: {info['endpoint']}",
                f"Category: {info['category']}",
                f"Help: {info.get('help', 'N/A')}",
                "",
                f"Total Fields: {info['total_fields']}",
                f"Required Fields: {info['required_fields_count']}",
                f"Fields with Defaults: {info['fields_with_defaults_count']}",
            ]
            if 'mkey' in info:
                lines.append(f"\nPrimary Key: {info['mkey']} ({info['mkey_type']})")
            return "\n".join(lines)
        
        # Field help
        meta = get_field_metadata(field_name)
        if meta is None:
            return f"Unknown field: {field_name}"

        lines = [
            f"Field: {meta['name']}",
            f"Type: {meta['type']}",
        ]
        if 'description' in meta:
            lines.append(f"Description: {meta['description']}")
        lines.append(f"Required: {'Yes' if meta.get('required', False) else 'No'}")
        if 'default' in meta:
            lines.append(f"Default: {meta['default']}")
        if 'options' in meta:
            lines.append(f"Options: {', '.join(meta['options'])}")
        if 'constraints' in meta:
            constraints = meta['constraints']
            if 'min' in constraints or 'max' in constraints:
                min_val = constraints.get('min', '?')
                max_val = constraints.get('max', '?')
                lines.append(f"Range: {min_val} - {max_val}")
            if 'max_length' in constraints:
                lines.append(f"Max Length: {constraints['max_length']}")

        return "\n".join(lines)

    @staticmethod
    def fields(detailed: bool = False) -> Union[list[str], dict[str, dict]]:
        """
        Get list of all field names or detailed field information.

        Args:
            detailed: If True, return dict with field metadata

        Returns:
            List of field names or dict of field metadata

        Examples:
            >>> # Simple list
            >>> fields = Interface.fields()
            >>> print(f"Available fields: {len(fields)}")
            
            >>> # Detailed info
            >>> fields = Interface.fields(detailed=True)
            >>> for name, meta in fields.items():
            ...     print(f"{name}: {meta['type']}")
        """
        from ._helpers.interface import get_all_fields, get_field_metadata

        field_names = get_all_fields()

        if not detailed:
            return field_names

        # Build detailed dict
        detailed_fields = {}
        for fname in field_names:
            meta = get_field_metadata(fname)
            if meta:
                detailed_fields[fname] = meta

        return detailed_fields

    @staticmethod
    def field_info(field_name: str) -> dict[str, Any] | None:
        """
        Get complete metadata for a specific field.

        Args:
            field_name: Name of the field

        Returns:
            Field metadata dict or None if field doesn't exist

        Examples:
            >>> info = Interface.field_info("name")
            >>> print(f"Type: {info['type']}")
            >>> if 'options' in info:
            ...     print(f"Options: {info['options']}")
        """
        from ._helpers.interface import get_field_metadata

        return get_field_metadata(field_name)

    @staticmethod
    def validate_field(field_name: str, value: Any) -> tuple[bool, str | None]:
        """
        Validate a field value against its constraints.

        Args:
            field_name: Name of the field
            value: Value to validate

        Returns:
            Tuple of (is_valid, error_message)

        Examples:
            >>> is_valid, error = Interface.validate_field("name", "test")
            >>> if not is_valid:
            ...     print(f"Validation error: {error}")
        """
        from ._helpers.interface import validate_field_value

        return validate_field_value(field_name, value)

    @staticmethod
    def required_fields() -> list[str]:
        """
        Get list of required field names.

        Note: Due to FortiOS schema quirks, some fields may be conditionally required.
        Always test with the actual API for authoritative requirements.

        Returns:
            List of required field names

        Examples:
            >>> required = Interface.required_fields()
            >>> print(f"Required fields: {', '.join(required)}")
        """
        from ._helpers.interface import REQUIRED_FIELDS

        return REQUIRED_FIELDS.copy()

    @staticmethod
    def defaults() -> dict[str, Any]:
        """
        Get all fields with default values.

        Returns:
            Dict mapping field names to default values

        Examples:
            >>> defaults = Interface.defaults()
            >>> print(f"Fields with defaults: {len(defaults)}")
            >>> # Use as starting point for payload
            >>> payload = defaults.copy()
            >>> payload['name'] = 'my-custom-name'
        """
        from ._helpers.interface import FIELDS_WITH_DEFAULTS

        return FIELDS_WITH_DEFAULTS.copy()

    @staticmethod
    def schema() -> dict[str, Any]:
        """
        Get complete schema information for this endpoint.

        Returns:
            Schema metadata dict containing endpoint info, field counts, and primary key

        Examples:
            >>> schema = Interface.schema()
            >>> print(f"Endpoint: {schema['endpoint']}")
            >>> print(f"Total fields: {schema['total_fields']}")
            >>> print(f"Primary key: {schema.get('mkey', 'N/A')}")
        """
        from ._helpers.interface import get_schema_info

        return get_schema_info()



from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class LocalPayload(TypedDict, total=False):
    """
    Type hints for user/local payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: LocalPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: NotRequired[str]  # Local user name.
    id: NotRequired[int]  # User ID.
    status: Literal["enable", "disable"]  # Enable/disable allowing the local user to authenticate with 
    type: Literal["password", "radius", "tacacs+", "ldap", "saml"]  # Authentication method.
    passwd: str  # User's password.
    ldap_server: str  # Name of LDAP server with which the user must authenticate.
    radius_server: str  # Name of RADIUS server with which the user must authenticate.
    tacacs+_server: str  # Name of TACACS+ server with which the user must authenticate
    saml_server: str  # Name of SAML server with which the user must authenticate.
    two_factor: NotRequired[Literal["disable", "fortitoken", "fortitoken-cloud", "email", "sms"]]  # Enable/disable two-factor authentication.
    two_factor_authentication: NotRequired[Literal["fortitoken", "email", "sms"]]  # Authentication method by FortiToken Cloud.
    two_factor_notification: NotRequired[Literal["email", "sms"]]  # Notification method for user activation by FortiToken Cloud.
    fortitoken: NotRequired[str]  # Two-factor recipient's FortiToken serial number.
    email_to: NotRequired[str]  # Two-factor recipient's email address.
    sms_server: NotRequired[Literal["fortiguard", "custom"]]  # Send SMS through FortiGuard or other external server.
    sms_custom_server: NotRequired[str]  # Two-factor recipient's SMS server.
    sms_phone: NotRequired[str]  # Two-factor recipient's mobile phone number.
    passwd_policy: NotRequired[str]  # Password policy to apply to this user, as defined in config 
    passwd_time: NotRequired[str]  # Time of the last password update.
    authtimeout: NotRequired[int]  # Time in minutes before the authentication timeout for a user
    workstation: NotRequired[str]  # Name of the remote user workstation, if you want to limit th
    auth_concurrent_override: NotRequired[Literal["enable", "disable"]]  # Enable/disable overriding the policy-auth-concurrent under c
    auth_concurrent_value: NotRequired[int]  # Maximum number of concurrent logins permitted from the same 
    ppk_secret: NotRequired[str]  # IKEv2 Postquantum Preshared Key (ASCII string or hexadecimal
    ppk_identity: NotRequired[str]  # IKEv2 Postquantum Preshared Key Identity.
    qkd_profile: NotRequired[str]  # Quantum Key Distribution (QKD) profile.
    username_sensitivity: NotRequired[Literal["disable", "enable"]]  # Enable/disable case and accent sensitivity when performing u


class Local:
    """
    Configure local users.
    
    Path: user/local
    Category: cmdb
    Primary Key: name
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: LocalPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        type: Literal["password", "radius", "tacacs+", "ldap", "saml"] | None = ...,
        passwd: str | None = ...,
        ldap_server: str | None = ...,
        radius_server: str | None = ...,
        tacacs+_server: str | None = ...,
        saml_server: str | None = ...,
        two_factor: Literal["disable", "fortitoken", "fortitoken-cloud", "email", "sms"] | None = ...,
        two_factor_authentication: Literal["fortitoken", "email", "sms"] | None = ...,
        two_factor_notification: Literal["email", "sms"] | None = ...,
        fortitoken: str | None = ...,
        email_to: str | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        sms_phone: str | None = ...,
        passwd_policy: str | None = ...,
        passwd_time: str | None = ...,
        authtimeout: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: LocalPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        type: Literal["password", "radius", "tacacs+", "ldap", "saml"] | None = ...,
        passwd: str | None = ...,
        ldap_server: str | None = ...,
        radius_server: str | None = ...,
        tacacs+_server: str | None = ...,
        saml_server: str | None = ...,
        two_factor: Literal["disable", "fortitoken", "fortitoken-cloud", "email", "sms"] | None = ...,
        two_factor_authentication: Literal["fortitoken", "email", "sms"] | None = ...,
        two_factor_notification: Literal["email", "sms"] | None = ...,
        fortitoken: str | None = ...,
        email_to: str | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        sms_phone: str | None = ...,
        passwd_policy: str | None = ...,
        passwd_time: str | None = ...,
        authtimeout: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: LocalPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "Local",
    "LocalPayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class VapPayload(TypedDict, total=False):
    """
    Type hints for wireless_controller/vap payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: VapPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: str  # Virtual AP name.
    pre_auth: NotRequired[Literal["enable", "disable"]]  # Enable/disable pre-authentication, where supported by client
    external_pre_auth: NotRequired[Literal["enable", "disable"]]  # Enable/disable pre-authentication with external APs not mana
    mesh_backhaul: NotRequired[Literal["enable", "disable"]]  # Enable/disable using this VAP as a WiFi mesh backhaul (defau
    atf_weight: NotRequired[int]  # Airtime weight in percentage (default = 20).
    max_clients: NotRequired[int]  # Maximum number of clients that can connect simultaneously to
    max_clients_ap: NotRequired[int]  # Maximum number of clients that can connect simultaneously to
    ssid: NotRequired[str]  # IEEE 802.11 service set identifier (SSID) for the wireless i
    broadcast_ssid: NotRequired[Literal["enable", "disable"]]  # Enable/disable broadcasting the SSID (default = enable).
    security: NotRequired[Literal["open", "wep64", "wep128", "wpa-personal", "wpa-enterprise", "wpa-only-personal", "wpa-only-enterprise", "wpa2-only-personal", "wpa2-only-enterprise", "wpa3-enterprise", "wpa3-only-enterprise", "wpa3-enterprise-transition", "wpa3-sae", "wpa3-sae-transition", "owe", "osen"]]  # Security mode for the wireless interface (default = wpa2-onl
    pmf: NotRequired[Literal["disable", "enable", "optional"]]  # Protected Management Frames (PMF) support (default = disable
    pmf_assoc_comeback_timeout: NotRequired[int]  # Protected Management Frames (PMF) comeback maximum timeout (
    pmf_sa_query_retry_timeout: NotRequired[int]  # Protected Management Frames (PMF) SA query retry timeout int
    beacon_protection: NotRequired[Literal["disable", "enable"]]  # Enable/disable beacon protection support (default = disable)
    okc: NotRequired[Literal["disable", "enable"]]  # Enable/disable Opportunistic Key Caching (OKC) (default = en
    mbo: NotRequired[Literal["disable", "enable"]]  # Enable/disable Multiband Operation (default = disable).
    gas_comeback_delay: NotRequired[int]  # GAS comeback delay (0 or 100 - 10000 milliseconds, default =
    gas_fragmentation_limit: NotRequired[int]  # GAS fragmentation limit (512 - 4096, default = 1024).
    mbo_cell_data_conn_pref: NotRequired[Literal["excluded", "prefer-not", "prefer-use"]]  # MBO cell data connection preference (0, 1, or 255, default =
    80211k: NotRequired[Literal["disable", "enable"]]  # Enable/disable 802.11k assisted roaming (default = enable).
    80211v: NotRequired[Literal["disable", "enable"]]  # Enable/disable 802.11v assisted roaming (default = enable).
    neighbor_report_dual_band: NotRequired[Literal["disable", "enable"]]  # Enable/disable dual-band neighbor report (default = disable)
    fast_bss_transition: NotRequired[Literal["disable", "enable"]]  # Enable/disable 802.11r Fast BSS Transition (FT) (default = d
    ft_mobility_domain: NotRequired[int]  # Mobility domain identifier in FT (1 - 65535, default = 1000)
    ft_r0_key_lifetime: NotRequired[int]  # Lifetime of the PMK-R0 key in FT, 1-65535 minutes.
    ft_over_ds: NotRequired[Literal["disable", "enable"]]  # Enable/disable FT over the Distribution System (DS).
    sae_groups: NotRequired[Literal["19", "20", "21"]]  # SAE-Groups.
    owe_groups: NotRequired[Literal["19", "20", "21"]]  # OWE-Groups.
    owe_transition: NotRequired[Literal["disable", "enable"]]  # Enable/disable OWE transition mode support.
    owe_transition_ssid: NotRequired[str]  # OWE transition mode peer SSID.
    additional_akms: NotRequired[Literal["akm6", "akm24"]]  # Additional AKMs.
    eapol_key_retries: NotRequired[Literal["disable", "enable"]]  # Enable/disable retransmission of EAPOL-Key frames (message 3
    tkip_counter_measure: NotRequired[Literal["enable", "disable"]]  # Enable/disable TKIP counter measure.
    external_web: NotRequired[str]  # URL of external authentication web server.
    external_web_format: NotRequired[Literal["auto-detect", "no-query-string", "partial-query-string"]]  # URL query parameter detection (default = auto-detect).
    external_logout: NotRequired[str]  # URL of external authentication logout server.
    mac_username_delimiter: NotRequired[Literal["hyphen", "single-hyphen", "colon", "none"]]  # MAC authentication username delimiter (default = hyphen).
    mac_password_delimiter: NotRequired[Literal["hyphen", "single-hyphen", "colon", "none"]]  # MAC authentication password delimiter (default = hyphen).
    mac_calling_station_delimiter: NotRequired[Literal["hyphen", "single-hyphen", "colon", "none"]]  # MAC calling station delimiter (default = hyphen).
    mac_called_station_delimiter: NotRequired[Literal["hyphen", "single-hyphen", "colon", "none"]]  # MAC called station delimiter (default = hyphen).
    mac_case: NotRequired[Literal["uppercase", "lowercase"]]  # MAC case (default = uppercase).
    called_station_id_type: NotRequired[Literal["mac", "ip", "apname"]]  # The format type of RADIUS attribute Called-Station-Id (defau
    mac_auth_bypass: NotRequired[Literal["enable", "disable"]]  # Enable/disable MAC authentication bypass.
    radius_mac_auth: NotRequired[Literal["enable", "disable"]]  # Enable/disable RADIUS-based MAC authentication of clients (d
    radius_mac_auth_server: NotRequired[str]  # RADIUS-based MAC authentication server.
    radius_mac_auth_block_interval: NotRequired[int]  # Don't send RADIUS MAC auth request again if the client has b
    radius_mac_mpsk_auth: NotRequired[Literal["enable", "disable"]]  # Enable/disable RADIUS-based MAC authentication of clients fo
    radius_mac_mpsk_timeout: NotRequired[int]  # RADIUS MAC MPSK cache timeout interval (0 or 300 - 864000, d
    radius_mac_auth_usergroups: NotRequired[list[dict[str, Any]]]  # Selective user groups that are permitted for RADIUS mac auth
    auth: NotRequired[Literal["radius", "usergroup"]]  # Authentication protocol.
    encrypt: NotRequired[Literal["TKIP", "AES", "TKIP-AES"]]  # Encryption protocol to use (only available when security is 
    keyindex: NotRequired[int]  # WEP key index (1 - 4).
    key: NotRequired[str]  # WEP Key.
    passphrase: NotRequired[str]  # WPA pre-shared key (PSK) to be used to authenticate WiFi use
    sae_password: NotRequired[str]  # WPA3 SAE password to be used to authenticate WiFi users.
    sae_h2e_only: NotRequired[Literal["enable", "disable"]]  # Use hash-to-element-only mechanism for PWE derivation (defau
    sae_hnp_only: NotRequired[Literal["enable", "disable"]]  # Use hunting-and-pecking-only mechanism for PWE derivation (d
    sae_pk: NotRequired[Literal["enable", "disable"]]  # Enable/disable WPA3 SAE-PK (default = disable).
    sae_private_key: NotRequired[str]  # Private key used for WPA3 SAE-PK authentication.
    akm24_only: NotRequired[Literal["disable", "enable"]]  # WPA3 SAE using group-dependent hash only (default = disable)
    radius_server: NotRequired[str]  # RADIUS server to be used to authenticate WiFi users.
    nas_filter_rule: NotRequired[Literal["enable", "disable"]]  # Enable/disable NAS filter rule support (default = disable).
    domain_name_stripping: NotRequired[Literal["disable", "enable"]]  # Enable/disable stripping domain name from identity (default 
    local_standalone: NotRequired[Literal["enable", "disable"]]  # Enable/disable AP local standalone (default = disable).
    local_standalone_nat: NotRequired[Literal["enable", "disable"]]  # Enable/disable AP local standalone NAT mode.
    ip: NotRequired[str]  # IP address and subnet mask for the local standalone NAT subn
    dhcp_lease_time: NotRequired[int]  # DHCP lease time in seconds for NAT IP address.
    local_standalone_dns: NotRequired[Literal["enable", "disable"]]  # Enable/disable AP local standalone DNS.
    local_standalone_dns_ip: NotRequired[str]  # IPv4 addresses for the local standalone DNS.
    local_lan_partition: NotRequired[Literal["enable", "disable"]]  # Enable/disable segregating client traffic to local LAN side 
    local_bridging: NotRequired[Literal["enable", "disable"]]  # Enable/disable bridging of wireless and Ethernet interfaces 
    local_lan: NotRequired[Literal["allow", "deny"]]  # Allow/deny traffic destined for a Class A, B, or C private I
    local_authentication: NotRequired[Literal["enable", "disable"]]  # Enable/disable AP local authentication.
    usergroup: NotRequired[list[dict[str, Any]]]  # Firewall user group to be used to authenticate WiFi users.
    captive_portal: NotRequired[Literal["enable", "disable"]]  # Enable/disable captive portal.
    captive_network_assistant_bypass: NotRequired[Literal["enable", "disable"]]  # Enable/disable Captive Network Assistant bypass.
    portal_message_override_group: NotRequired[str]  # Replacement message group for this VAP (only available when 
    portal_message_overrides: NotRequired[str]  # Individual message overrides.
    portal_type: NotRequired[Literal["auth", "auth+disclaimer", "disclaimer", "email-collect", "cmcc", "cmcc-macauth", "auth-mac", "external-auth", "external-macauth"]]  # Captive portal functionality. Configure how the captive port
    selected_usergroups: NotRequired[list[dict[str, Any]]]  # Selective user groups that are permitted to authenticate.
    security_exempt_list: NotRequired[str]  # Optional security exempt list for captive portal authenticat
    security_redirect_url: NotRequired[str]  # Optional URL for redirecting users after they pass captive p
    auth_cert: NotRequired[str]  # HTTPS server certificate.
    auth_portal_addr: NotRequired[str]  # Address of captive portal.
    intra_vap_privacy: NotRequired[Literal["enable", "disable"]]  # Enable/disable blocking communication between clients on the
    schedule: NotRequired[list[dict[str, Any]]]  # Firewall schedules for enabling this VAP on the FortiAP. Thi
    ldpc: NotRequired[Literal["disable", "rx", "tx", "rxtx"]]  # VAP low-density parity-check (LDPC) coding configuration.
    high_efficiency: NotRequired[Literal["enable", "disable"]]  # Enable/disable 802.11ax high efficiency (default = enable).
    target_wake_time: NotRequired[Literal["enable", "disable"]]  # Enable/disable 802.11ax target wake time (default = enable).
    port_macauth: NotRequired[Literal["disable", "radius", "address-group"]]  # Enable/disable LAN port MAC authentication (default = disabl
    port_macauth_timeout: NotRequired[int]  # LAN port MAC authentication idle timeout value (default = 60
    port_macauth_reauth_timeout: NotRequired[int]  # LAN port MAC authentication re-authentication timeout value 
    bss_color_partial: NotRequired[Literal["enable", "disable"]]  # Enable/disable 802.11ax partial BSS color (default = enable)
    mpsk_profile: NotRequired[str]  # MPSK profile name.
    split_tunneling: NotRequired[Literal["enable", "disable"]]  # Enable/disable split tunneling (default = disable).
    nac: NotRequired[Literal["enable", "disable"]]  # Enable/disable network access control.
    nac_profile: NotRequired[str]  # NAC profile name.
    vlanid: NotRequired[int]  # Optional VLAN ID.
    vlan_auto: NotRequired[Literal["enable", "disable"]]  # Enable/disable automatic management of SSID VLAN interface.
    dynamic_vlan: NotRequired[Literal["enable", "disable"]]  # Enable/disable dynamic VLAN assignment.
    captive_portal_fw_accounting: NotRequired[Literal["enable", "disable"]]  # Enable/disable RADIUS accounting for captive portal firewall
    captive_portal_ac_name: NotRequired[str]  # Local-bridging captive portal ac-name.
    captive_portal_auth_timeout: NotRequired[int]  # Hard timeout - AP will always clear the session after timeou
    multicast_rate: NotRequired[Literal["0", "6000", "12000", "24000"]]  # Multicast rate (0, 6000, 12000, or 24000 kbps, default = 0).
    multicast_enhance: NotRequired[Literal["enable", "disable"]]  # Enable/disable converting multicast to unicast to improve pe
    igmp_snooping: NotRequired[Literal["enable", "disable"]]  # Enable/disable IGMP snooping.
    dhcp_address_enforcement: NotRequired[Literal["enable", "disable"]]  # Enable/disable DHCP address enforcement (default = disable).
    broadcast_suppression: NotRequired[Literal["dhcp-up", "dhcp-down", "dhcp-starvation", "dhcp-ucast", "arp-known", "arp-unknown", "arp-reply", "arp-poison", "arp-proxy", "netbios-ns", "netbios-ds", "ipv6", "all-other-mc", "all-other-bc"]]  # Optional suppression of broadcast messages. For example, you
    ipv6_rules: NotRequired[Literal["drop-icmp6ra", "drop-icmp6rs", "drop-llmnr6", "drop-icmp6mld2", "drop-dhcp6s", "drop-dhcp6c", "ndp-proxy", "drop-ns-dad", "drop-ns-nondad"]]  # Optional rules of IPv6 packets. For example, you can keep RA
    me_disable_thresh: NotRequired[int]  # Disable multicast enhancement when this many clients are rec
    mu_mimo: NotRequired[Literal["enable", "disable"]]  # Enable/disable Multi-user MIMO (default = enable).
    probe_resp_suppression: NotRequired[Literal["enable", "disable"]]  # Enable/disable probe response suppression (to ignore weak si
    probe_resp_threshold: NotRequired[str]  # Minimum signal level/threshold in dBm required for the AP re
    radio_sensitivity: NotRequired[Literal["enable", "disable"]]  # Enable/disable software radio sensitivity (to ignore weak si
    quarantine: NotRequired[Literal["enable", "disable"]]  # Enable/disable station quarantine (default = enable).
    radio_5g_threshold: NotRequired[str]  # Minimum signal level/threshold in dBm required for the AP re
    radio_2g_threshold: NotRequired[str]  # Minimum signal level/threshold in dBm required for the AP re
    vlan_name: NotRequired[list[dict[str, Any]]]  # Table for mapping VLAN name to VLAN ID.
    vlan_pooling: NotRequired[Literal["wtp-group", "round-robin", "hash", "disable"]]  # Enable/disable VLAN pooling, to allow grouping of multiple w
    vlan_pool: NotRequired[list[dict[str, Any]]]  # VLAN pool.
    dhcp_option43_insertion: NotRequired[Literal["enable", "disable"]]  # Enable/disable insertion of DHCP option 43 (default = enable
    dhcp_option82_insertion: NotRequired[Literal["enable", "disable"]]  # Enable/disable DHCP option 82 insert (default = disable).
    dhcp_option82_circuit_id_insertion: NotRequired[Literal["style-1", "style-2", "style-3", "disable"]]  # Enable/disable DHCP option 82 circuit-id insert (default = d
    dhcp_option82_remote_id_insertion: NotRequired[Literal["style-1", "disable"]]  # Enable/disable DHCP option 82 remote-id insert (default = di
    ptk_rekey: NotRequired[Literal["enable", "disable"]]  # Enable/disable PTK rekey for WPA-Enterprise security.
    ptk_rekey_intv: NotRequired[int]  # PTK rekey interval (600 - 864000 sec, default = 86400).
    gtk_rekey: NotRequired[Literal["enable", "disable"]]  # Enable/disable GTK rekey for WPA security.
    gtk_rekey_intv: NotRequired[int]  # GTK rekey interval (600 - 864000 sec, default = 86400).
    eap_reauth: NotRequired[Literal["enable", "disable"]]  # Enable/disable EAP re-authentication for WPA-Enterprise secu
    eap_reauth_intv: NotRequired[int]  # EAP re-authentication interval (1800 - 864000 sec, default =
    roaming_acct_interim_update: NotRequired[Literal["enable", "disable"]]  # Enable/disable using accounting interim update instead of ac
    qos_profile: NotRequired[str]  # Quality of service profile name.
    hotspot20_profile: NotRequired[str]  # Hotspot 2.0 profile name.
    access_control_list: NotRequired[str]  # Profile name for access-control-list.
    primary_wag_profile: NotRequired[str]  # Primary wireless access gateway profile name.
    secondary_wag_profile: NotRequired[str]  # Secondary wireless access gateway profile name.
    tunnel_echo_interval: NotRequired[int]  # The time interval to send echo to both primary and secondary
    tunnel_fallback_interval: NotRequired[int]  # The time interval for secondary tunnel to fall back to prima
    rates_11a: NotRequired[Literal["6", "6-basic", "9", "9-basic", "12", "12-basic", "18", "18-basic", "24", "24-basic", "36", "36-basic", "48", "48-basic", "54", "54-basic"]]  # Allowed data rates for 802.11a.
    rates_11bg: NotRequired[Literal["1", "1-basic", "2", "2-basic", "5.5", "5.5-basic", "11", "11-basic", "6", "6-basic", "9", "9-basic", "12", "12-basic", "18", "18-basic", "24", "24-basic", "36", "36-basic", "48", "48-basic", "54", "54-basic"]]  # Allowed data rates for 802.11b/g.
    rates_11n_ss12: NotRequired[Literal["mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/2", "mcs9/2", "mcs10/2", "mcs11/2", "mcs12/2", "mcs13/2", "mcs14/2", "mcs15/2"]]  # Allowed data rates for 802.11n with 1 or 2 spatial streams.
    rates_11n_ss34: NotRequired[Literal["mcs16/3", "mcs17/3", "mcs18/3", "mcs19/3", "mcs20/3", "mcs21/3", "mcs22/3", "mcs23/3", "mcs24/4", "mcs25/4", "mcs26/4", "mcs27/4", "mcs28/4", "mcs29/4", "mcs30/4", "mcs31/4"]]  # Allowed data rates for 802.11n with 3 or 4 spatial streams.
    rates_11ac_mcs_map: NotRequired[str]  # Comma separated list of max supported VHT MCS for spatial st
    rates_11ax_mcs_map: NotRequired[str]  # Comma separated list of max supported HE MCS for spatial str
    rates_11be_mcs_map: NotRequired[str]  # Comma separated list of max nss that supports EHT-MCS 0-9, 1
    rates_11be_mcs_map_160: NotRequired[str]  # Comma separated list of max nss that supports EHT-MCS 0-9, 1
    rates_11be_mcs_map_320: NotRequired[str]  # Comma separated list of max nss that supports EHT-MCS 0-9, 1
    utm_profile: NotRequired[str]  # UTM profile name.
    utm_status: NotRequired[Literal["enable", "disable"]]  # Enable to add one or more security profiles (AV, IPS, etc.) 
    utm_log: NotRequired[Literal["enable", "disable"]]  # Enable/disable UTM logging.
    ips_sensor: NotRequired[str]  # IPS sensor name.
    application_list: NotRequired[str]  # Application control list name.
    antivirus_profile: NotRequired[str]  # AntiVirus profile name.
    webfilter_profile: NotRequired[str]  # WebFilter profile name.
    scan_botnet_connections: NotRequired[Literal["disable", "monitor", "block"]]  # Block or monitor connections to Botnet servers or disable Bo
    address_group: NotRequired[str]  # Firewall Address Group Name.
    address_group_policy: NotRequired[Literal["disable", "allow", "deny"]]  # Configure MAC address filtering policy for MAC addresses tha
    sticky_client_remove: NotRequired[Literal["enable", "disable"]]  # Enable/disable sticky client remove to maintain good signal 
    sticky_client_threshold_5g: NotRequired[str]  # Minimum signal level/threshold in dBm required for the 5G cl
    sticky_client_threshold_2g: NotRequired[str]  # Minimum signal level/threshold in dBm required for the 2G cl
    sticky_client_threshold_6g: NotRequired[str]  # Minimum signal level/threshold in dBm required for the 6G cl
    bstm_rssi_disassoc_timer: NotRequired[int]  # Time interval for client to voluntarily leave AP before forc
    bstm_load_balancing_disassoc_timer: NotRequired[int]  # Time interval for client to voluntarily leave AP before forc
    bstm_disassociation_imminent: NotRequired[Literal["enable", "disable"]]  # Enable/disable forcing of disassociation after the BSTM requ
    beacon_advertising: NotRequired[Literal["name", "model", "serial-number"]]  # Fortinet beacon advertising IE data   (default = empty).
    osen: NotRequired[Literal["enable", "disable"]]  # Enable/disable OSEN as part of key management (default = dis
    application_detection_engine: NotRequired[Literal["enable", "disable"]]  # Enable/disable application detection engine (default = disab
    application_dscp_marking: NotRequired[Literal["enable", "disable"]]  # Enable/disable application attribute based DSCP marking (def
    application_report_intv: NotRequired[int]  # Application report interval (30 - 864000 sec, default = 120)
    l3_roaming: NotRequired[Literal["enable", "disable"]]  # Enable/disable layer 3 roaming (default = disable).
    l3_roaming_mode: NotRequired[Literal["direct", "indirect"]]  # Select the way that layer 3 roaming traffic is passed (defau


class Vap:
    """
    Configure Virtual Access Points (VAPs).
    
    Path: wireless_controller/vap
    Category: cmdb
    Primary Key: name
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: VapPayload | None = ...,
        name: str | None = ...,
        pre_auth: Literal["enable", "disable"] | None = ...,
        external_pre_auth: Literal["enable", "disable"] | None = ...,
        mesh_backhaul: Literal["enable", "disable"] | None = ...,
        atf_weight: int | None = ...,
        max_clients: int | None = ...,
        max_clients_ap: int | None = ...,
        ssid: str | None = ...,
        broadcast_ssid: Literal["enable", "disable"] | None = ...,
        security: Literal["open", "wep64", "wep128", "wpa-personal", "wpa-enterprise", "wpa-only-personal", "wpa-only-enterprise", "wpa2-only-personal", "wpa2-only-enterprise", "wpa3-enterprise", "wpa3-only-enterprise", "wpa3-enterprise-transition", "wpa3-sae", "wpa3-sae-transition", "owe", "osen"] | None = ...,
        pmf: Literal["disable", "enable", "optional"] | None = ...,
        pmf_assoc_comeback_timeout: int | None = ...,
        pmf_sa_query_retry_timeout: int | None = ...,
        beacon_protection: Literal["disable", "enable"] | None = ...,
        okc: Literal["disable", "enable"] | None = ...,
        mbo: Literal["disable", "enable"] | None = ...,
        gas_comeback_delay: int | None = ...,
        gas_fragmentation_limit: int | None = ...,
        mbo_cell_data_conn_pref: Literal["excluded", "prefer-not", "prefer-use"] | None = ...,
        80211k: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: VapPayload | None = ...,
        name: str | None = ...,
        pre_auth: Literal["enable", "disable"] | None = ...,
        external_pre_auth: Literal["enable", "disable"] | None = ...,
        mesh_backhaul: Literal["enable", "disable"] | None = ...,
        atf_weight: int | None = ...,
        max_clients: int | None = ...,
        max_clients_ap: int | None = ...,
        ssid: str | None = ...,
        broadcast_ssid: Literal["enable", "disable"] | None = ...,
        security: Literal["open", "wep64", "wep128", "wpa-personal", "wpa-enterprise", "wpa-only-personal", "wpa-only-enterprise", "wpa2-only-personal", "wpa2-only-enterprise", "wpa3-enterprise", "wpa3-only-enterprise", "wpa3-enterprise-transition", "wpa3-sae", "wpa3-sae-transition", "owe", "osen"] | None = ...,
        pmf: Literal["disable", "enable", "optional"] | None = ...,
        pmf_assoc_comeback_timeout: int | None = ...,
        pmf_sa_query_retry_timeout: int | None = ...,
        beacon_protection: Literal["disable", "enable"] | None = ...,
        okc: Literal["disable", "enable"] | None = ...,
        mbo: Literal["disable", "enable"] | None = ...,
        gas_comeback_delay: int | None = ...,
        gas_fragmentation_limit: int | None = ...,
        mbo_cell_data_conn_pref: Literal["excluded", "prefer-not", "prefer-use"] | None = ...,
        80211k: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: VapPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "Vap",
    "VapPayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class PeerPayload(TypedDict, total=False):
    """
    Type hints for user/peer payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: PeerPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: NotRequired[str]  # Peer name.
    mandatory_ca_verify: NotRequired[Literal["enable", "disable"]]  # Determine what happens to the peer if the CA certificate is 
    ca: NotRequired[str]  # Name of the CA certificate.
    subject: NotRequired[str]  # Peer certificate name constraints.
    cn: NotRequired[str]  # Peer certificate common name.
    cn_type: NotRequired[Literal["string", "email", "FQDN", "ipv4", "ipv6"]]  # Peer certificate common name type.
    mfa_mode: NotRequired[Literal["none", "password", "subject-identity"]]  # MFA mode for remote peer authentication/authorization.
    mfa_server: NotRequired[str]  # Name of a remote authenticator. Performs client access right
    mfa_username: NotRequired[str]  # Unified username for remote authentication.
    mfa_password: NotRequired[str]  # Unified password for remote authentication. This field may b
    ocsp_override_server: NotRequired[str]  # Online Certificate Status Protocol (OCSP) server for certifi
    two_factor: NotRequired[Literal["enable", "disable"]]  # Enable/disable two-factor authentication, applying certifica
    passwd: NotRequired[str]  # Peer's password used for two-factor authentication.


class Peer:
    """
    Configure peer users.
    
    Path: user/peer
    Category: cmdb
    Primary Key: name
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: PeerPayload | None = ...,
        name: str | None = ...,
        mandatory_ca_verify: Literal["enable", "disable"] | None = ...,
        ca: str | None = ...,
        subject: str | None = ...,
        cn: str | None = ...,
        cn_type: Literal["string", "email", "FQDN", "ipv4", "ipv6"] | None = ...,
        mfa_mode: Literal["none", "password", "subject-identity"] | None = ...,
        mfa_server: str | None = ...,
        mfa_username: str | None = ...,
        mfa_password: str | None = ...,
        ocsp_override_server: str | None = ...,
        two_factor: Literal["enable", "disable"] | None = ...,
        passwd: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: PeerPayload | None = ...,
        name: str | None = ...,
        mandatory_ca_verify: Literal["enable", "disable"] | None = ...,
        ca: str | None = ...,
        subject: str | None = ...,
        cn: str | None = ...,
        cn_type: Literal["string", "email", "FQDN", "ipv4", "ipv6"] | None = ...,
        mfa_mode: Literal["none", "password", "subject-identity"] | None = ...,
        mfa_server: str | None = ...,
        mfa_username: str | None = ...,
        mfa_password: str | None = ...,
        ocsp_override_server: str | None = ...,
        two_factor: Literal["enable", "disable"] | None = ...,
        passwd: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: PeerPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "Peer",
    "PeerPayload",
]